<?php
/**
 * @package     CSVI
 * @subpackage  Form2Content
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die;

jimport('joomla.form.helper');
JFormHelper::loadFieldClass('CsviForm');

/**
 * Select list form field with content types
 *
 * @package     CSVI
 * @subpackage  Form2Content
 * @since       7.1.0
 */
class Csviform2contentFormFieldContentType extends JFormFieldCsviForm
{
	/**
	 * Method to instantiate the form field object.
	 *
	 * @param   JForm  $form  The form to attach to the form field object.
	 *
	 * @since   7.1.0
	 *
	 * @throws  Exception
	 */
	public function __construct($form = null)
	{
		$this->type = 'ContentType';

		parent::__construct($form);
	}

	/**
	 * Select content types.
	 *
	 * @return  array  An array of content types.
	 *
	 * @since   7.1.0
	 *
	 * @throws  Exception
	 * @throws  CsviException
	 * @throws  RuntimeException
	 *
	 */
	protected function getOptions()
	{
		$query = $this->db->getQuery(true)
			->select(
				array(
					$this->db->quoteName('id', 'value'),
					$this->db->quoteName('title', 'text'))
			)
			->from($this->db->quoteName('#__f2c_project'))
			->order($this->db->quoteName('text'))
			->group($this->db->quoteName('value'));
		$this->db->setQuery($query);
		$contentTypes = $this->db->loadObjectList();

		if (!$contentTypes)
		{
			$contentTypes = array();
		}

		return array_merge(parent::getOptions(), $contentTypes);
	}
}
