<?php
/**
 * @package     CSVI
 * @subpackage  Form2Content
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace form2content\com_form2content\model\export;

defined('_JEXEC') or die;

/**
 * Export Form2Content form.
 *
 * @package     CSVI
 * @subpackage  Form2Content
 * @since       7.0
 */
class F2carticle extends \CsviModelExports
{
	/**
	 * List of custom fields
	 *
	 * @var    array
	 * @since  7.0
	 */
	private $contentFields = array();

	/**
	 * The Form2Content helper
	 *
	 * @var    \Com_Form2ContentHelperCom_Form2Content
	 * @since  7.0
	 */
	protected $helper = null;

	/**
	 * Export the data.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	protected function exportBody()
	{
		if (parent::exportBody())
		{
			// Build something fancy to only get the fieldnames the user wants
			$exportFields = $this->fields->getFields();
			$this->loadContentFields();

			// Group by fields
			$groupFields = json_decode($this->template->get('groupbyfields', '', 'string'));
			$groupBy       = array();

			// Group articles by id as default
			$groupBy[] = $this->db->quoteName('f.id');

			if (isset($groupFields->name))
			{
				$groupByFields = array_flip($groupFields->name);
			}
			else
			{
				$groupByFields = array();
			}

			// Sort selected fields
			$sortFields = json_decode($this->template->get('sortfields', '', 'string'));
			$sortBy     = array();

			if (isset($sortFields->name))
			{
				$sortByFields = array_flip($sortFields->name);
			}
			else
			{
				$sortByFields = array();
			}

			// Fields which are needed for getting contents
			$userFields = array();
			$userFields[] = $this->db->quoteName('fc.formid');
			$userFields[] = $this->db->quoteName('f.projectid');

			foreach ($exportFields as $field)
			{
				switch ($field->field_name)
				{
					case 'category_path':
						$userFields[] = $this->db->quoteName('f.catid');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('f.catid');
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('f.catid');
						}
						break;
					case 'projecttitle':
						$userFields[] = $this->db->quoteName('f.projectid');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('f.projectid');
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('f.projectid');
						}
						break;
					case 'asset_id':
					case 'projectid':
					case 'created_by':
					case 'created_by_alias':
					case 'created':
					case 'modified':
					case 'id':
					case 'metadata':
					case 'metadesc':
					case 'metakey':
					case 'title':
					case 'alias':
					case 'version':
					case 'state':
					case 'intro_template':
					case 'main_template':
					case 'reference_id':
					case 'ordering':
						$userFields[] = $this->db->quoteName('f.' . $field->field_name);

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('f.' . $field->field_name);
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('f.' . $field->field_name);
						}
						break;
					case 'fieldtitle':
					case 'fieldsettings':
						$userFields[] = $this->db->quoteName('pf.projectid');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('pf.projectid');
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('pf.projectid');
						}
						break;
					case 'fieldtypename':
						$userFields[] = $this->db->quoteName('pf.fieldtypeid');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('pf.fieldtypeid');
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('pf.fieldtypeid');
						}
						break;
					case 'fieldname':
					case 'fieldtypeid':
					case 'frontvisible':
						$userFields[] = $this->db->quoteName('pf.' . $field->field_name);

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('pf.' . $field->field_name);
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('pf.' . $field->field_name);
						}
						break;
					case 'formid':
					case 'fieldid':
					case 'attribute':
					case 'content':
						$userFields[] = $this->db->quoteName('fc.' . $field->field_name);

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('fc.' . $field->field_name);
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('fc.' . $field->field_name);
						}
						break;
					case 'show_title':
					case 'link_titles':
					case 'show_intro':
					case 'show_category':
					case 'link_category':
					case 'show_parent_category':
					case 'link_parent_category':
					case 'show_author':
					case 'link_author':
					case 'show_create_date':
					case 'show_modify_date':
					case 'show_publish_date':
					case 'show_item_navigation':
					case 'show_icons':
					case 'show_print_icon':
					case 'show_email_icon':
					case 'show_vote':
					case 'show_hits':
					case 'show_noauth':
					case 'urls_position':
					case 'alternative_readmore':
					case 'article_layout':
					case 'show_publishing_options':
					case 'show_article_options':
					case 'show_urls_images_backend':
					case 'show_urls_images_frontend':
						$userFields[] = $this->db->quoteName('f.id');
						$userFields[] = $this->db->quoteName('f.attribs');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('f.attribs');
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('f.attribs');
						}
						break;
					case 'tags':
						$userFields[] = $this->db->quoteName('f.extended');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('f.extended');
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('f.extended');
						}
						break;
					case 'robots':
					case 'metaauthor':
					case 'rights':
					case 'xreference':
						$userFields[] = $this->db->quoteName('f.metadata');

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName('f.metadata');
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName('f.metadata');
						}
						break;
					case 'custom':
					case 'picture_url':
						break;
					default:
						if (!in_array($field->field_name, $this->contentFields))
						{
							$userFields[] = $this->db->quoteName($field->field_name);

							if (array_key_exists($field->field_name, $groupByFields))
							{
								$groupBy[] = $this->db->quoteName($field->field_name);
							}

							if (array_key_exists($field->field_name, $sortByFields))
							{
								$sortBy[] = $this->db->quoteName($field->field_name);
							}
						}

						break;
				}
			}

			// Build the query
			$userFields = array_unique($userFields);
			$query      = $this->db->getQuery(true);
			$query->select(implode(",\n", $userFields));
			$query->from($this->db->quoteName('#__f2c_form', 'f'));
			$query->leftJoin(
				$this->db->quoteName('#__f2c_projectfields', 'pf') . ' ON ' .
				$this->db->quoteName('pf.projectid') . ' = ' . $this->db->quoteName('f.projectid')
			);
			$query->leftJoin(
				$this->db->quoteName('#__f2c_fieldcontent', 'fc') . ' ON ' .
				$this->db->quoteName('fc.formid') . ' = ' . $this->db->quoteName('f.id')
			);

			// Filter by article category
			$articleCategory = $this->template->get('articlecategories');

			if ($articleCategory && $articleCategory[0] !== '*')
			{
				if ($this->template->get('inclsubcategory', false))
				{
					foreach ($articleCategory as $category)
					{
						$subCategoryIds = $this->helper->getSubCategoryIds($category);

						if ($subCategoryIds)
						{
							$articleCategory = array_merge($articleCategory, $subCategoryIds);
						}
					}
				}

				$query->where($this->db->quoteName('f.catid') . " IN ('" . implode("','", $articleCategory) . "')");
			}

			// Filter by content type
			$contentType = $this->template->get('contenttype');

			if ($contentType && $contentType[0] !== 'none')
			{
				$query->where($this->db->quoteName('f.projectid') . " IN ('" . implode("','", $contentType) . "')");
			}

			// Group the fields
			$groupBy = array_unique($groupBy);

			if (0 !== count($groupBy))
			{
				$query->group($groupBy);
			}

			// Sort set fields
			$sortBy = array_unique($sortBy);

			if (0 !== count($sortBy))
			{
				$query->order($sortBy);
			}

			// Add export limits
			$limits = $this->getExportLimit();

			// Execute the query
			$this->db->setQuery($query, $limits['offset'], $limits['limit']);
			$records = $this->db->getIterator();
			$this->log->add('Export query' . $query->__toString(), false);

			// Check if there are any records
			$logCount = $this->db->getNumRows();

			if ($logCount > 0)
			{
				foreach ($records as $record)
				{
					$this->log->incrementLinenumber();

					// Clean some settings
					$attribs  = '';
					$extended = '';
					$metaData = '';

					foreach ($exportFields as $field)
					{
						$fieldName = $field->field_name;

						// Set the field value
						if (isset($record->$fieldName))
						{
							$fieldValue = $record->$fieldName;
						}
						else
						{
							$fieldValue = '';
						}

						// Process the field
						switch ($fieldName)
						{
							case 'projecttitle':
								$query->clear()
									->select($this->db->quoteName('title'))
									->from($this->db->quoteName('#__f2c_project'))
									->where($this->db->quoteName('id') . ' = ' . (int) $record->projectid);
								$this->db->setQuery($query);
								$fieldValue = $this->db->loadResult();
								break;
							case 'category_path':
								$query->clear()
									->select($this->db->quoteName('path'))
									->from($this->db->quoteName('#__categories'))
									->where($this->db->quoteName('id') . ' = ' . (int) $record->catid);
								$this->db->setQuery($query);
								$fieldValue = $this->db->loadResult();
								break;
							case 'fieldtitle':
								$query->clear()
									->select($this->db->quoteName('title'))
									->from($this->db->quoteName('#__f2c_projectfields'))
									->where($this->db->quoteName('id') . ' = ' . (int) $record->projectid);
								$this->db->setQuery($query);
								$fieldValue = $this->db->loadResult();
								break;
							case 'fieldsettings':
								$query->clear()
									->select($this->db->quoteName('settings'))
									->from($this->db->quoteName('#__f2c_projectfields'))
									->where($this->db->quoteName('id') . ' = ' . (int) $record->projectid);
								$this->db->setQuery($query);
								$fieldValue = $this->db->loadResult();
								break;
							case 'fieldtypename':
								$query->clear()
									->select($this->db->quoteName('name'))
									->from($this->db->quoteName('#__f2c_fieldtype'))
									->where($this->db->quoteName('id') . ' = ' . (int) $record->fieldtypeid);
								$this->db->setQuery($query);
								$fieldValue = $this->db->loadResult();
								break;
							case 'show_title':
							case 'link_titles':
							case 'show_intro':
							case 'show_category':
							case 'link_category':
							case 'show_parent_category':
							case 'link_parent_category':
							case 'show_author':
							case 'link_author':
							case 'show_create_date':
							case 'show_modify_date':
							case 'show_publish_date':
							case 'show_item_navigation':
							case 'show_icons':
							case 'show_print_icon':
							case 'show_email_icon':
							case 'show_vote':
							case 'show_hits':
							case 'show_noauth':
							case 'urls_position':
							case 'alternative_readmore':
							case 'article_layout':
							case 'show_publishing_options':
							case 'show_article_options':
							case 'show_urls_images_backend':
							case 'show_urls_images_frontend':
								if (empty($attribs))
								{
									$attribs = json_decode($record->attribs);
								}

								if (isset($attribs->$fieldName))
								{
									$fieldValue = $attribs->$fieldName;
								}
								break;
							case 'tags':
								if (empty($extended))
								{
									$extended = json_decode($record->extended);
								}

								if (isset($extended->$fieldName))
								{
									$fieldValue = $extended->$fieldName;
								}
								break;
							case 'robots':
							case 'metaauthor':
							case 'rights':
							case 'xreference':
								if ($fieldName === 'metaauthor')
								{
									$fieldName = 'author';
								}

								if (empty($metaData))
								{
									$metaData = json_decode($record->metadata);
								}

								if (isset($metaData->$fieldName))
								{
									$fieldValue = $metaData->$fieldName;
								}
								break;
							case 'picture_url':
								$fieldValue = '';
								$domainName = $this->settings->get('hostname');
								$formId     = $record->formid;
								$fieldId    = $this->helper->getFieldId('Image', $record->projectid);

								if ($fieldId && $formId)
								{
									$query = $this->db->getQuery(true)
										->select($this->db->quoteName('content'))
										->from($this->db->quoteName('#__f2c_fieldcontent'))
										->where($this->db->quoteName('formid') . '=' . (int) $formId)
										->where($this->db->quoteName('fieldid') . '=' . (int) $fieldId);
									$this->db->setQuery($query);
									$fieldValueResult = $this->db->loadObjectList();

									if ($fieldValueResult)
									{
										$imageArray    = json_decode($fieldValueResult[0]->content, true);
										$params        = \JComponentHelper::getParams('com_form2content');
										$imageBasePath = $params->get('image_path', '/images/stories/com_form2content');
										$imagePath     = $imageBasePath . '/p' . $record->projectid . '/f' . $formId . '/';

										if ($imageArray)
										{
											foreach ($imageArray as $arrayName => $image)
											{
												if ($arrayName === 'filename')
												{
													$fieldValue = $domainName . $imagePath . $image;
												}
											}
										}
									}
								}
								break;
							default:
								if (in_array($fieldName, $this->contentFields))
								{
									$formId             = $record->formid;
									$fieldId            = $this->helper->getFieldId($fieldName, $record->projectid);
									$fieldValue         = '';
									$fieldValueResult   = '';

									if ($fieldId && $formId)
									{
										$query = $this->db->getQuery(true)
											->select($this->db->quoteName('content'))
											->from($this->db->quoteName('#__f2c_fieldcontent'))
											->where($this->db->quoteName('formid') . '=' . (int) $formId)
											->where($this->db->quoteName('fieldid') . '=' . (int) $fieldId)
											->order($this->db->quoteName('id'));
										$this->db->setQuery($query);
										$fieldValueResult = $this->db->loadObjectList();
									}

									$query->clear()
										->select($this->db->quoteName('name'))
										->select($this->db->quoteName('settings'))
										->from($this->db->quoteName('#__f2c_fieldtype', 'ft'))
										->leftJoin(
											$this->db->quoteName('#__f2c_projectfields', 'pf') . ' ON ' .
											$this->db->quoteName('pf.fieldtypeid') . ' = ' . $this->db->quoteName('ft.id')
										)
										->where($this->db->quoteName('pf.fieldname') . '=' . $this->db->quote($fieldName))
										->where($this->db->quoteName('pf.projectid') . '=' . (int) $record->projectid);
									$this->db->setQuery($query);
									$fieldTypes = $this->db->loadObject();

									if ($fieldTypes)
									{
										switch (strtolower($fieldTypes->name))
										{
											case 'databaselookup':
											case 'databaselookupmulti':
												if ($fieldValueResult)
												{
													$fieldValueArray = array();

													$query->clear()
														->select($this->db->quoteName('title'))
														->from($this->db->quoteName('#__f2c_form'));

													foreach ($fieldValueResult as $index => $item)
													{
														$query->clear('where')
															->where($this->db->quoteName('reference_id') . ' = ' . (int) $item->content);
														$this->db->setQuery($query);
														$fieldValueArray[] = $this->db->loadResult();
													}

													$fieldValue = implode('|', $fieldValueArray);
												}

												break;
											case 'displaylist':
												if ($fieldValueResult)
												{
													$displayArray = json_decode($fieldValueResult[0]->content, true);

													if (!empty($displayArray))
													{
														$fieldValue = implode('|', $displayArray);
													}
												}
												break;
											case 'image':
												if ($fieldValueResult)
												{
													$imageFinalArray = array();
													$imageArray      = json_decode($fieldValueResult[0]->content, true);

													if (!empty($imageArray))
													{
														foreach ($imageArray as $arrayName => $image)
														{
															$imageFinalArray[] = $arrayName . ':' . $image;
														}

														$fieldValue = implode('|', $imageFinalArray);
													}
												}

												break;
											default:
												if ($fieldValueResult)
												{
													$fieldValue = $fieldValueResult[0]->content;
												}
												break;
										}
									}
								}
								break;
						}

						// Store the field value
						$this->fields->set($field->csvi_templatefield_id, $fieldValue);
					}

					// Output the data
					$this->addExportFields();

					// Output the contents
					$this->writeOutput();
				}
			}
			else
			{
				$this->addExportContent(\JText::_('COM_CSVI_NO_DATA_FOUND'));

				// Output the contents
				$this->writeOutput();
			}
		}
	}

	/**
	 * Get a list of content fields that can be used as available field.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 *
	 * @throws  \Exception
	 */
	private function loadContentFields()
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('fieldname'))
			->from($this->db->quoteName('#__f2c_projectfields'))
			->group($this->db->quoteName('fieldname'));
		$this->db->setQuery($query);

		$result = $this->db->loadColumn();

		if (!is_array($result))
		{
			$result = array();
		}

		$this->contentFields = $result;
	}
}
