<?php
/**
 * @package     CSVI
 * @subpackage  Form2Content
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

/**
 * Form2Content maintenance.
 *
 * @package     CSVI
 * @subpackage  Form2Content
 * @since       7.0
 */
class Com_Form2ContentMaintenance
{
	/**
	 * Database connector
	 *
	 * @var    JDatabaseDriver
	 * @since  7.0
	 */
	private $db = null;

	/**
	 * Logger helper
	 *
	 * @var    CsviHelperLog
	 * @since  7.0
	 */
	private $log = null;

	/**
	 * CSVI Helper.
	 *
	 * @var    CsviHelperCsvi
	 * @since  7.0
	 */
	private $csvihelper = null;

	/**
	 * Constructor.
	 *
	 * @param   JDatabaseDriver  $db          The database class
	 * @param   CsviHelperLog    $log         The CSVI logger
	 * @param   CsviHelperCsvi   $csvihelper  The CSVI helper
	 * @param   bool             $isCli       Set if we are running CLI mode
	 *
	 * @since   7.0
	 */
	public function __construct(JDatabaseDriver $db, CsviHelperLog $log, CsviHelperCsvi $csvihelper, $isCli = false)
	{
		$this->db         = $db;
		$this->log        = $log;
		$this->csvihelper = $csvihelper;
		$this->isCli      = $isCli;
	}

	/**
	 * Update available fields that require extra processing.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 *
	 * @throws  \RuntimeException
	 */
	public function customAvailableFields()
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('fieldname'))
			->from($this->db->quoteName('#__f2c_projectfields'))
			->group($this->db->quoteName('fieldname'));
		$this->db->setQuery($query);

		$fieldName = $this->db->loadColumn();

		// Start the query
		$query->clear()
			->insert($this->db->quoteName('#__csvi_availablefields'))
			->columns($this->db->quoteName(array('csvi_name', 'component_name', 'component_table', 'component', 'action')));

		if (count($fieldName) > 0)
		{
			foreach ($fieldName as $field)
			{
				$query->values(
					$this->db->quote($field) . ',' .
					$this->db->quote($field) . ',' .
					$this->db->quote('f2c_form') . ',' .
					$this->db->quote('com_form2content') . ',' .
					$this->db->quote('export')
				);

				$query->values(
					$this->db->quote($field) . ',' .
					$this->db->quote($field) . ',' .
					$this->db->quote('f2c_form') . ',' .
					$this->db->quote('com_form2content') . ',' .
					$this->db->quote('import')
				);

			}

			$this->db->setQuery($query)->execute();
		}
	}

	/**
	 * Threshold available fields for extension
	 *
	 * @return  int Hardcoded available fields
	 *
	 * @since   7.0
	 */
	public function availableFieldsThresholdLimit()
	{
		return 160;
	}
}

