<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

jimport('joomla.form.helper');
JFormHelper::loadFieldClass('CsviForm');

/**
 * Select list form field with currencies.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.5.0
 */
class JFormFieldCsviHikashopOrderCurrencies extends JFormFieldCsviForm
{
	/**
	 * Type of field
	 *
	 * @var    string
	 * @since  6.5.0
	 */
	protected $type = 'CsviHikashopOrderCurrencies';

	/**
	 * Get the options.
	 *
	 * @return  array  An array of Currency for orders.
	 *
	 * @since   6.5.0
	 */
	protected function getOptions()
	{
		$q = $this->db->getQuery(true)
			->select(
				array(
					$this->db->quoteName('o.order_currency_id', 'value'),
					$this->db->quoteName('c.currency_name', 'text'))
			)
			->from($this->db->quoteName('#__hikashop_order', 'o'))
			->leftJoin(
				$this->db->quoteName('#__hikashop_currency', 'c')
				. ' ON ' . $this->db->quoteName('c.currency_id') . ' = ' . $this->db->quoteName('o.order_currency_id')
			)
			->order($this->db->quoteName('text'))
			->group($this->db->quoteName('value'));

		$this->db->setQuery($q);
		$currencies = $this->db->loadObjectList();

		if (empty($currencies))
		{
			$currencies = array();
		}

		return array_merge(parent::getOptions(), $currencies);
	}
}
