<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

jimport('joomla.form.helper');
JFormHelper::loadFieldClass('CsviForm');

/**
 * Select list form field for payment.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.5.0
 */
class JFormFieldCsviHikashopOrderPayment extends JFormFieldCsviForm
{
	/**
	 * Type of field
	 *
	 * @var    string
	 * @since  6.5.0
	 */
	protected $type = 'CsviHikashopOrderPayment';

	/**
	 * Get the options.
	 *
	 * @return  array  An array of payment methods.
	 *
	 * @since   6.5.0
	 */
	protected function getOptions()
	{
		$query = $this->db->getQuery(true)
			->select(
				array(
					$this->db->quoteName('p.payment_id', 'value'),
					$this->db->quoteName('p.payment_name', 'text'))
			)
			->from($this->db->quoteName('#__hikashop_payment', 'p'))
			->rightJoin(
				$this->db->quoteName('#__hikashop_order', 'o')
				. ' ON ' . $this->db->quoteName('o.order_payment_id') . ' = ' . $this->db->quoteName('p.payment_id')
			)
			->order($this->db->quoteName('text'))
			->group($this->db->quoteName('value'));

		$this->db->setQuery($query);
		$payments = $this->db->loadObjectList();

		if (empty($payments))
		{
			$payments = array();
		}

		return array_merge(parent::getOptions(), $payments);
	}
}
