<?php
/**
 * @package     CSVI
 * @subpackage  Hikashop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

jimport('joomla.form.helper');
JFormHelper::loadFieldClass('CsviForm');

/**
 * Select list form field with order products.
 *
 * @package     CSVI
 * @subpackage  Hikashop
 * @since       6.5.0
 */
class JFormFieldCsviHikashopOrderProduct extends JFormFieldCsviForm
{
	/**
	 * Type of field
	 *
	 * @var    string
	 * @since  6.5.0
	 */
	protected $type = 'CsviHikashopOrderProduct';

	/**
	 * Select order products.
	 *
	 * @return  array  An array of products.
	 *
	 * @since   6.5.0
	 */
	protected function getOptions()
	{
		$ids = $this->form->getValue('orderproduct', 'jform');

		if (!empty($ids) && !empty($ids[0]))
		{
			$query = $this->db->getQuery(true)
				->select($this->db->quoteName('order_product_code', 'value') . ',' . $this->db->quoteName('order_product_name', 'text'))
				->from($this->db->quoteName('#__hikashop_order_product'))
				->where($this->db->quoteName('order_product_code') . ' IN (' . implode(',', $this->db->quote($ids)) . ')')
				->order($this->db->quoteName('order_product_code'))
				->group($this->db->quoteName('order_product_code'));

			$this->db->setQuery($query);
			$orderproducts = $this->db->loadObjectList();

			if (empty($orderproducts))
			{
				$orderproducts = array();
			}

			return array_merge(parent::getOptions(), $orderproducts);
		}
		else
		{
			return parent::getOptions();
		}
	}
}
