<?php
/**
 * @package     CSVI
 * @subpackage  Hikashop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

jimport('joomla.form.helper');
JFormHelper::loadFieldClass('CsviForm');

/**
 * Select list form field with order users.
 *
 * @package     CSVI
 * @subpackage  Hikashop
 * @since       6.5.0
 */
class JFormFieldCsviHikashopOrderUser extends JFormFieldCsviForm
{
	/**
	 * Type of field
	 *
	 * @var    string
	 * @since  6.5.0
	 */
	protected $type = 'CsviHikashopOrderUser';

	/**
	 * Select order users.
	 *
	 * @return  array  An array of users.
	 *
	 * @since   6.5.0
	 */
	protected function getOptions()
	{
		$userids = $this->form->getValue('orderuser', 'jform');

		if (!empty($userids) && !empty($userids[0]))
		{
			$query = $this->db->getQuery(true)
				->select(
					array(
						$this->db->quoteName('ho.order_user_id', 'value'),
						$this->db->quoteName('u.name', 'text'))
				)
				->from($this->db->quoteName('#__hikashop_order', 'ho'))
				->leftJoin(
					$this->db->quoteName('#__hikashop_user', 'hu')
					. ' ON ' . $this->db->quoteName('hu.user_id') . ' = ' . $this->db->quoteName('ho.order_user_id')
				)
				->leftJoin(
					$this->db->quoteName('#__users', 'u')
					. ' ON ' . $this->db->quoteName('u.id') . ' = ' . $this->db->quoteName('hu.user_cms_id')
				)
				->order($this->db->quoteName('text'))
				->group($this->db->quoteName('value'));

			$this->db->setQuery($query);
			$customers = $this->db->loadObjectList();

			if (empty($customers))
			{
				$customers = array();
			}

			return array_merge(parent::getOptions(), $customers);
		}
		else
		{
			return parent::getOptions();
		}
	}
}
