<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

/**
 * The HikaShop Config Class.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.0
 */
class Com_HikashopHelperCom_Hikashop_Config
{
	/**
	 * Holds the HikaShop configuration
	 *
	 * @var    JRegistry
	 * @since  4.0
	 */
	private $cfg = array();

	/**
	 * Constructor.
	 *
	 * @since  4.0
	 */
	public function __construct()
	{
		$this->parse();
	}

	/**
	 * Finds a given HikaShop setting.
	 *
	 * @param   string  $setting  The config value to find.
	 *
	 * @return  mixed  Value if found | False if not found.
	 *
	 * @since   4.0
	 */
	public function get($setting)
	{
		return $this->cfg->get($setting . '.config_value', false);
	}

	/**
	 * Parse the HikaShop configuration.
	 *
	 * @return  void.
	 *
	 * @since   4.0
	 */
	private function parse()
	{
		$db = JFactory::getDbo();
		$query = $db->getQuery(true)
			->select($db->quoteName('config_namekey') . ',' . $db->quoteName('config_value'))
			->from($db->quoteName('#__hikashop_config'));
		$db->setQuery($query);
		$params = $db->loadAssocList('config_namekey');

		$this->cfg = new JRegistry;
		$this->cfg->loadArray($params);
	}
}
