<?php
/**
 * @package     CSVI
 * @subpackage  Hikashop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

/**
 * The Hikashop helper class.
 *
 * @package     CSVI
 * @subpackage  Hikashop
 * @since       6.5.0
 */
class Com_HikashopHelperCom_Hikashop_Json
{
	/**
	 * Database connector
	 *
	 * @var    JDatabase
	 * @since   6.5.0
	 */
	protected $db = null;

	/**
	 * Constructor.
	 *
	 * @param   JDatabase  $db  Database connector.
	 *
	 * @since    6.5.0
	 */
	public function __construct(JDatabase $db)
	{
		$this->db = $db;
	}

	/**
	 * Get a filtered list of order users.
	 *
	 * @param   string  $filter  The filter to use.
	 *
	 * @return  array  List of order users matching filter.
	 *
	 * @since    6.5.0
	 */
	public function getOrderUser($filter)
	{
		$query = $this->db->getQuery(true)
			->select(
				array(
					$this->db->quoteName('ho.order_user_id', 'user_id'),
					$this->db->quoteName('u.name', 'user_name'))
			)
			->from($this->db->quoteName('#__hikashop_order', 'ho'))
			->leftJoin(
				$this->db->quoteName('#__hikashop_user', 'hu')
				. ' ON ' . $this->db->quoteName('hu.user_id') . ' = ' . $this->db->quoteName('ho.order_user_id')
			)
			->leftJoin(
				$this->db->quoteName('#__users', 'u')
				. ' ON ' . $this->db->quoteName('u.id') . ' = ' . $this->db->quoteName('hu.user_cms_id')
			)
			->where($this->db->quoteName('u.name') . ' LIKE ' . $this->db->quote('%' . $filter . '%'))
			->order($this->db->quoteName('user_name'))
			->group($this->db->quoteName('user_id'));

		$this->db->setQuery($query, 0, 10);

		return $this->db->loadObjectList();
	}

	/**
	 * Get a filtered list of order products.
	 *
	 * @param   string  $filter  The filter to use.
	 *
	 * @return  array  List of order users matching filter.
	 *
	 * @since    6.5.0
	 */
	public function getOrderProduct($filter)
	{
		$query = $this->db->getQuery(true)
			->select(
				$this->db->quoteName('order_product_id', 'product_id') . ',
				IF (LENGTH(TRIM(' . $this->db->quoteName('order_product_code') . ')) = 0, '
					. $this->db->quote(JText::_('COM_CSVI_EXPORT_ORDER_SKU_EMPTY')) . ', '
					. $this->db->quoteName('order_product_code')
				. ') AS ' . $this->db->quoteName('product_sku') . ', ' .
				$this->db->quoteName('order_product_name', 'product_name')
			)
			->from($this->db->quoteName('#__hikashop_order_product'))
			->where($this->db->quoteName('order_product_code') . ' LIKE ' . $this->db->quote('%' . $filter . '%'), 'OR')
			->where($this->db->quoteName('order_product_name') . ' LIKE ' . $this->db->quote('%' . $filter . '%'))
			->order($this->db->quoteName('order_product_name'))
			->group($this->db->quoteName('order_product_code'));

		$this->db->setQuery($query, 0, 10);

		return $this->db->loadObjectList();
	}
}
