<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace hikashop\com_hikashop\model\export;

defined('_JEXEC') or die;

use Joomla\Utilities\ArrayHelper;


/**
 * Exports HikaShop custom fields.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.6.0
 */
class Customfield extends \CsviModelExports
{
	/**
	 * Field Display options
	 *
	 * @var    array
	 * @since  6.6.0
	 */
	private $fieldDisplay = array();

	/**
	 * Field Options options
	 *
	 * @var    array
	 * @since  6.6.0
	 */
	private $fieldOptions = array();

	/**
	 * Export the data.
	 *
	 * @return  bool  True if body is exported | False if body is not exported.
	 *
	 * @since   6.6.0
	 *
	 * @throws  \CsviException
	 * @throws  \RuntimeException
	 */
	protected function exportBody()
	{
		if (parent::exportBody())
		{
			// Get some basic data
			require_once JPATH_PLUGINS . '/csviaddon/hikashop/com_hikashop/helper/com_hikashop_category_product.php';
			$categoryHelper = new \Com_HikashopHelperCom_Hikashop_Category_Product($this->db, $this->template, $this->log);
			$exportfields = $this->fields->getFields();

			// Group by fields
			$groupbyfields = json_decode($this->template->get('groupbyfields', '', 'string'));
			$groupby = array();

			if (isset($groupbyfields->name))
			{
				$groupbyfields = array_flip($groupbyfields->name);
			}
			else
			{
				$groupbyfields = array();
			}

			// Sort selected fields
			$sortfields = json_decode($this->template->get('sortfields', '', 'string'));
			$sortby = array();

			if (isset($sortfields->name))
			{
				$sortbyfields = array_flip($sortfields->name);
			}
			else
			{
				$sortbyfields = array();
			}

			// Build something fancy to only get the fieldnames the user wants
			$userfields = array();

			foreach ($exportfields as $field)
			{
				switch ($field->field_name)
				{
					case 'category_path':
						$userfields[] = $this->db->quoteName('f.field_categories');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('f.field_categories');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('f.field_categories');
						}
						break;
					case 'usergroup_name':
						$userfields[] = $this->db->quoteName('f.field_access');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('f.field_access');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('f.field_access');
						}
						break;
					case 'field_item_show_cart':
					case 'field_item_back_cart_details':
					case 'field_item_checkout':
					case 'field_item_front_order':
					case 'field_item_front_product_listing':
					case 'field_item_product_show':
					case 'field_item_product_cart':
					case 'field_item_order_form':
					case 'field_item_back_invoice':
					case 'field_item_back_shipping_invoice':
					case 'field_item_edit_product_order':
					case 'field_item_mail_order_notif':
					case 'field_item_mail_status_notif':
					case 'field_item_mail_order_creation':
					case 'field_item_mail_admin_notif':
					case 'field_item_mail_payment_notif':
					case 'field_item_front_cart_details':
					case 'field_item_order_edit':
					case 'field_product_compare':
					case 'field_product_listing':
					case 'field_product_front_listing':
					case 'field_product_back_invoice':
					case 'field_product_back_shipping_invoice':
					case 'field_product_order_form':
					case 'field_product_back_cart_details':
					case 'field_product_front_cart_details':
					case 'field_product_mail_order_notif':
					case 'field_product_mail_status_notif':
					case 'field_product_mail_order_creation':
					case 'field_product_mail_admin_notif':
					case 'field_product_mail_payment_notif':
					case 'field_order_front_order':
					case 'field_order_invoice':
					case 'field_order_back_shipping_invoice':
					case 'field_order_edit':
					case 'field_order_show':
					case 'field_order_checkout':
					case 'field_order_listing':
					case 'field_order_form':
					case 'field_order_edit_fields':
					case 'field_order_mail_order_notif':
					case 'field_order_mail_status_notif':
					case 'field_order_mail_order_creation':
					case 'field_order_mail_admin_notif':
					case 'field_order_mail_payment_notif':
						$userfields[] = $this->db->quoteName('f.field_display');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('f.field_display');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('f.field_display');
						}
						break;
					case 'errormessage':
					case 'regex':
					case 'attribute':
					case 'placeholder':
					case 'cols':
					case 'filtering':
					case 'maxlength':
					case 'rows':
					case 'zone_type':
					case 'pleaseselect':
					case 'size':
					case 'format':
					case 'allow':
					case 'readonly':
					case 'translatable':
					case 'limit_to_parent':
					case 'parent_value':
						$userfields[] = $this->db->quoteName('f.field_options');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('f.field_options');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('f.field_options');
						}
						break;
					case 'product_code':
					case 'product_id':
					case 'product_value':
						$userfields[] = $this->db->quoteName('f.field_products');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('f.field_products');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('f.field_products');
						}
						break;
					case 'custom':
						break;
					default:
						$userfields[] = $this->db->quoteName($field->field_name);

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName($field->field_name);
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName($field->field_name);
						}
						break;
				}
			}

			/**
			 * Export SQL Query
			 * Get all products - including items
			 * as well as products without a price
			 */
			$userfields = array_unique($userfields);
			$query = $this->db->getQuery(true);
			$query->select(implode(",\n", $userfields));
			$query->from($this->db->quoteName('#__hikashop_field', 'f'));

			// Group the fields
			$groupby = array_unique($groupby);

			if (!empty($groupby))
			{
				$query->group($groupby);
			}

			// Sort set fields
			$sortby = array_unique($sortby);

			if (!empty($sortby))
			{
				$query->order($sortby);
			}

			// Add export limits
			$limits = $this->getExportLimit();

			// Execute the query
			$this->db->setQuery($query, $limits['offset'], $limits['limit']);
			$records = $this->db->getIterator();
			$this->log->add('Export query' . $query->__toString(), false);

			// Check if there are any records
			$logcount = $this->db->getNumRows();

			if ($logcount > 0)
			{
				foreach ($records as $record)
				{
					$this->log->incrementLinenumber();

					// Reset for the next record
					$this->fieldDisplay = array();
					$this->fieldOptions = array();

					// Process all the export fields
					foreach ($exportfields as $field)
					{
						// Get the field name
						$fieldName  = $field->field_name;
						$fieldValue = '';

						// Set the field value
						if (isset($record->$fieldName))
						{
							$fieldValue = $record->$fieldName;
						}

						// Process the field
						switch ($fieldName)
						{
							case 'category_path':
								$fieldValue = $record->field_categories;

								if ($record->field_categories !== 'all')
								{
									$categoryIds = ArrayHelper::toInteger(explode(',', $record->field_categories));

									$fieldValue = $categoryHelper->createCategoryPathById($categoryIds);
								}
								break;
							case 'field_value':
								$fieldValue = preg_replace("/\r|\n/", '|', $record->field_value);
								break;
							case 'usergroup_name':
								$fieldValue = $record->field_access;

								if ($record->field_access !== 'all')
								{
									$productIds = explode(',', $record->field_access);

									if ($productIds)
									{
										$productIds = ArrayHelper::toInteger($productIds);

										$query = $this->db->getQuery(true)
											->select($this->db->quoteName('title'))
											->from($this->db->quoteName('#__usergroups'))
											->where($this->db->quoteName('id') . ' IN (' . implode(',', $productIds) . ')');
										$this->db->setQuery($query);
										$codes = $this->db->loadColumn();
										$this->log->add('Usergroup name');

										$fieldValue = implode('|', $codes);
									}
								}
								break;
							case 'field_item_show_cart':
							case 'field_item_back_cart_details':
							case 'field_item_checkout':
							case 'field_item_front_order':
							case 'field_item_front_product_listing':
							case 'field_item_product_show':
							case 'field_item_product_cart':
							case 'field_item_order_form':
							case 'field_item_back_invoice':
							case 'field_item_back_shipping_invoice':
							case 'field_item_edit_product_order':
							case 'field_item_mail_order_notif':
							case 'field_item_mail_status_notif':
							case 'field_item_mail_order_creation':
							case 'field_item_mail_admin_notif':
							case 'field_item_mail_payment_notif':
							case 'field_item_front_cart_details':
							case 'field_item_order_edit':
							case 'field_product_compare':
							case 'field_product_listing':
							case 'field_product_front_listing':
							case 'field_product_back_invoice':
							case 'field_product_back_shipping_invoice':
							case 'field_product_order_form':
							case 'field_product_back_cart_details':
							case 'field_product_front_cart_details':
							case 'field_product_mail_order_notif':
							case 'field_product_mail_status_notif':
							case 'field_product_mail_order_creation':
							case 'field_product_mail_admin_notif':
							case 'field_product_mail_payment_notif':
							case 'field_order_front_order':
							case 'field_order_invoice':
							case 'field_order_back_shipping_invoice':
							case 'field_order_edit':
							case 'field_order_show':
							case 'field_order_checkout':
							case 'field_order_listing':
							case 'field_order_form':
							case 'field_order_edit_fields':
							case 'field_order_mail_order_notif':
							case 'field_order_mail_status_notif':
							case 'field_order_mail_order_creation':
							case 'field_order_mail_admin_notif':
							case 'field_order_mail_payment_notif':
								// Get the HikaShop name
								$firstPosition  = strpos($fieldName, '_');
								$secondPosition = strpos($fieldName, '_', $firstPosition + 1);
								$hikashopName   = substr($fieldName, $secondPosition + 1);

								if (false !== strpos($record->field_display, $hikashopName))
								{
									if (0 === count($this->fieldDisplay))
									{
										$data = explode(';', $record->field_display);

										foreach ($data as $dataValue)
										{
											if (false !== strpos($dataValue, '='))
											{
												list($displayName, $displayValue) = explode('=', $dataValue);

												$this->fieldDisplay[$displayName] = $displayValue;
											}
										}
									}

									if (array_key_exists($hikashopName, $this->fieldDisplay))
									{
										$fieldValue = $this->fieldDisplay[$hikashopName];

										if (is_numeric($fieldValue) && (int) $fieldValue === 0)
										{
											$fieldValue = 'N';
										}
										elseif (is_numeric($fieldValue) && (int) $fieldValue === 1)
										{
											$fieldValue = 'Y';
										}
									}
								}
								break;
							case 'errormessage':
							case 'regex':
							case 'attribute':
							case 'placeholder':
							case 'cols':
							case 'filtering':
							case 'maxlength':
							case 'rows':
							case 'zone_type':
							case 'pleaseselect':
							case 'size':
							case 'format':
							case 'allow':
							case 'readonly':
							case 'translatable':
							case 'limit_to_parent':
							case 'parent_value':
								if (false !== strpos($record->field_options, $fieldName))
								{
									if (0 === count($this->fieldOptions))
									{
										$this->fieldDisplay = unserialize($record->field_options);
									}

									if (array_key_exists($fieldName, $this->fieldDisplay))
									{
										switch ($fieldName)
										{
											case 'allow':
												$fieldValue = $this->fieldDisplay[$fieldName];

												if ('' === $fieldValue)
												{
													$fieldValue = 'all';
												}
												break;
											default:
												$fieldValue = $this->fieldDisplay[$fieldName];
												break;
										}
									}
								}
								break;
							case 'product_code':
								$productIds = explode(',', $record->field_products);

								if ($productIds)
								{
									$productIds = ArrayHelper::toInteger($productIds);

									$query = $this->db->getQuery(true)
										->select($this->db->quoteName('product_code'))
										->from($this->db->quoteName('#__hikashop_product'))
										->where($this->db->quoteName('product_id') . ' IN (' . implode(',', $productIds) . ')');
									$this->db->setQuery($query);
									$codes = $this->db->loadColumn();
									$this->log->add('Find the product codes');

									$fieldValue = implode('|', $codes);
								}
								break;
						}

						// Store the field value
						$this->fields->set($field->csvi_templatefield_id, $fieldValue);
					}

					// Output the data
					$this->addExportFields();

					// Output the contents
					$this->writeOutput();
				}
			}
			else
			{
				$this->addExportContent(\JText::_('COM_CSVI_NO_DATA_FOUND'));

				// Output the contents
				$this->writeOutput();
			}
		}
	}
}
