<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace hikashop\com_hikashop\model\export;

defined('_JEXEC') or die;

use Joomla\Utilities\ArrayHelper;


/**
 * Exports HikaShop discounts.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.3.0
 */
class Discount extends \CsviModelExports
{
	/**
	 * The HikaShop config helper
	 *
	 * @var    \Com_HikashopHelperCom_Hikashop_Config
	 * @since  6.0
	 */
	protected $helperConfig = null;

	/**
	 * Export the data.
	 *
	 * @return  bool  True if body is exported | False if body is not exported.
	 *
	 * @since   6.0
	 */
	protected function exportBody()
	{
		if (parent::exportBody())
		{
			// Get some basic data
			$this->domainname = $this->settings->get('hostname');
			require_once JPATH_PLUGINS . '/csviaddon/hikashop/com_hikashop/helper/com_hikashop_category_product.php';
			$categoryHelper = new \Com_HikashopHelperCom_Hikashop_Category_Product($this->db, $this->template, $this->log);
			$exportfields = $this->fields->getFields();

			// Group by fields
			$groupbyfields = json_decode($this->template->get('groupbyfields', '', 'string'));
			$groupby = array();

			if (isset($groupbyfields->name))
			{
				$groupbyfields = array_flip($groupbyfields->name);
			}
			else
			{
				$groupbyfields = array();
			}

			// Sort selected fields
			$sortfields = json_decode($this->template->get('sortfields', '', 'string'));
			$sortby = array();

			if (isset($sortfields->name))
			{
				$sortbyfields = array_flip($sortfields->name);
			}
			else
			{
				$sortbyfields = array();
			}

			// Build something fancy to only get the fieldnames the user wants
			$userfields   = array();

			foreach ($exportfields as $field)
			{
				switch ($field->field_name)
				{
					case 'category_path':
						$userfields[] = $this->db->quoteName('d.discount_product_id');
						$userfields[] = $this->db->quoteName('d.discount_category_id');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('d.discount_product_id');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('d.discount_product_id');
						}
						break;
					case 'tax_name':
						$userfields[] = $this->db->quoteName('d.discount_tax_id');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('d.discount_tax_id');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('d.discount_tax_id');
						}
						break;
					case 'currency_code':
						$userfields[] = $this->db->quoteName('d.discount_currency_id');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('d.discount_currency_id');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('d.discount_currency_id');
						}
						break;
					case 'usergroup_name':
						$userfields[] = $this->db->quoteName('d.discount_access');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('d.discount_access');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('d.discount_access');
						}
						break;
					case 'state_name':
					case 'country_name':
						$userfields[] = $this->db->quoteName('d.discount_zone_id');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('d.discount_zone_id');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('d.discount_zone_id');
						}
						break;
					case 'product_code':
						$userfields[] = $this->db->quoteName('d.discount_product_id');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('d.discount_product_id');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('d.discount_product_id');
						}
						break;
					case 'custom':
						break;
					default:
						$userfields[] = $this->db->quoteName('d.' . $field->field_name);

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('d.' . $field->field_name);
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('d.' . $field->field_name);
						}
						break;
				}
			}

			// Export SQL Query
			$userfields = array_unique($userfields);
			$query = $this->db->getQuery(true)
				->select(implode(",\n", $userfields))
				->from($this->db->quoteName('#__hikashop_discount', 'd'));

			// Group the fields
			$groupby = array_unique($groupby);

			if (!empty($groupby))
			{
				$query->group($groupby);
			}

			// Sort set fields
			$sortby = array_unique($sortby);

			if (!empty($sortby))
			{
				$query->order($sortby);
			}

			// Add export limits
			$limits = $this->getExportLimit();

			// Execute the query
			$this->db->setQuery($query, $limits['offset'], $limits['limit']);
			$records = $this->db->getIterator();
			$this->log->add('Export query' . $query->__toString(), false);

			// Check if there are any records
			$logcount = $this->db->getNumRows();

			if ($logcount > 0)
			{
				foreach ($records as $record)
				{
					$this->log->incrementLinenumber();

					// Reset the prices
					$this->prices = array();

					// Process all the export fields
					foreach ($exportfields as $field)
					{
						// Get the field name
						$fieldname = $field->field_name;

						// Set the field value
						if (isset($record->$fieldname))
						{
							$fieldvalue = $record->$fieldname;
						}
						else
						{
							$fieldvalue = '';
						}

						// Process the field
						switch ($fieldname)
						{
							case 'category_path':
								$categoryIds = explode(',', $record->discount_category_id);

								if ($categoryIds)
								{
									$categoryIds = ArrayHelper::toInteger($categoryIds);
									$fieldvalue = $categoryHelper->createCategoryPathById($categoryIds);
								}

								if (strlen(trim($fieldvalue)) == 0)
								{
									$fieldvalue = $field->default_value;
								}
								break;
							case 'currency_code':
								$query = $this->db->getQuery(true)
									->select($this->db->quoteName('currency_code'))
									->from($this->db->quoteName('#__hikashop_currency'))
									->where($this->db->quoteName('currency_id') . ' = ' . (int) $record->discount_currency_id);
								$this->db->setQuery($query);

								$fieldvalue = $this->db->loadResult();
								break;
							case 'usergroup_name':
								$userIds = explode(',', $record->discount_access);

								if ($userIds)
								{
									$userIds = ArrayHelper::toInteger($userIds);

									$query = $this->db->getQuery(true)
										->select($this->db->quoteName('title'))
										->from($this->db->quoteName('#__usergroups'))
										->where($this->db->quoteName('id') . ' IN (' . implode(',', $userIds) . ')');
									$this->db->setQuery($query);
									$titles = $this->db->loadColumn();
									$this->log->add('Usergroup name');

									$fieldvalue = implode('|', $titles);
								}
								break;
							case 'tax_name':
								$fieldvalue = $categoryHelper->createCategoryPathById($record->discount_tax_id);
								break;
							case 'state_name':
								$zones = explode(',', $record->discount_zone_id);
								$query = $this->db->getQuery(true)
									->select($this->db->quoteName('zone_name_english'))
									->from($this->db->quoteName('#__hikashop_zone'))
									->where($this->db->quoteName('zone_namekey') . ' IN (' . implode(',', $this->db->quote($zones)) . ')')
									->where($this->db->quoteName('zone_type') . ' = ' . $this->db->quote('state'));
								$this->db->setQuery($query);
								$states = $this->db->loadColumn();
								$this->log->add('State name');
								$fieldvalue = implode('|', $states);
								break;
							case 'country_name':
								$zones = explode(',', $record->discount_zone_id);
								$query = $this->db->getQuery(true)
									->select($this->db->quoteName('zone_name_english'))
									->from($this->db->quoteName('#__hikashop_zone'))
									->where($this->db->quoteName('zone_namekey') . ' IN (' . implode(',', $this->db->quote($zones)) . ')')
									->where($this->db->quoteName('zone_type') . ' = ' . $this->db->quote('country'));
								$this->db->setQuery($query);
								$countries = $this->db->loadColumn();
								$this->log->add('Country name');
								$fieldvalue = implode('|', $countries);
								break;
							case 'product_code':
								$productIds = explode(',', $record->discount_product_id);
								$productIds = ArrayHelper::toInteger($productIds);

								if ($productIds)
								{
									$query = $this->db->getQuery(true)
										->select($this->db->quoteName('product_code'))
										->from($this->db->quoteName('#__hikashop_product'))
										->where($this->db->quoteName('product_id') . ' IN (' . implode(',', $this->db->quote($productIds)) . ')');
									$this->db->setQuery($query);
									$codes = $this->db->loadColumn();
									$this->log->add('Country name');
									$fieldvalue = implode('|', $codes);
								}

								if (strlen(trim($fieldvalue)) == 0)
								{
									$fieldvalue = $field->default_value;
								}
								break;
							case 'discount_start':
							case 'discount_end':
								$fieldvalue = $this->fields->getDateFormat($fieldname, $record->$fieldname, $field->column_header);

								if (strlen(trim($fieldvalue)) == 0)
								{
									$fieldvalue = $field->default_value;
								}
								break;
							case 'discount_flat_amount':
							case 'discount_minimum_order':
							case 'discount_percent_amount':
								$fieldvalue = number_format(
									$fieldvalue,
									$this->template->get('export_price_format_decimal', 2, 'int'),
									$this->template->get('export_price_format_decsep'),
									$this->template->get('export_price_format_thousep')
								);

								if (strlen(trim($fieldvalue)) == 0)
								{
									$fieldvalue = $field->default_value;
								}
								break;
						}

						// Store the field value
						$this->fields->set($field->csvi_templatefield_id, $fieldvalue);
					}

					// Output the data
					$this->addExportFields();

					// Output the contents
					$this->writeOutput();
				}
			}
			else
			{
				$this->addExportContent(\JText::_('COM_CSVI_NO_DATA_FOUND'));

				// Output the contents
				$this->writeOutput();
			}
		}
	}
}
