<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace hikashop\com_hikashop\model\export;

defined('_JEXEC') or die;

use Joomla\Utilities\ArrayHelper;


/**
 * Exports HikaShop shipping rates.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.0
 */
class Shippingrate extends \CsviModelExports
{
	/**
	 * Export the data.
	 *
	 * @return  bool  True if body is exported | False if body is not exported.
	 *
	 * @since   6.0
	 *
	 * @throws  \CsviException
	 * @throws  \RuntimeException
	 */
	protected function exportBody()
	{
		if (parent::exportBody())
		{
			// Get some basic data
			require_once JPATH_PLUGINS . '/csviaddon/hikashop/com_hikashop/helper/com_hikashop_category_product.php';
			$categoryHelper = new \Com_HikashopHelperCom_Hikashop_Category_Product($this->db, $this->template, $this->log);
			$exportfields = $this->fields->getFields();

			// Group by fields
			$groupbyfields = json_decode($this->template->get('groupbyfields', '', 'string'));
			$groupby = array();

			if (isset($groupbyfields->name))
			{
				$groupbyfields = array_flip($groupbyfields->name);
			}
			else
			{
				$groupbyfields = array();
			}

			// Sort selected fields
			$sortfields = json_decode($this->template->get('sortfields', '', 'string'));
			$sortby = array();

			if (isset($sortfields->name))
			{
				$sortbyfields = array_flip($sortfields->name);
			}
			else
			{
				$sortbyfields = array();
			}

			// Build something fancy to only get the fieldnames the user wants
			$userfields = array();

			foreach ($exportfields as $field)
			{
				switch ($field->field_name)
				{
					case 'shipping_percentage':
					case 'shipping_per_product':
					case 'shipping_price_per_product':
					case 'shipping_override_address':
					case 'shipping_override_address_text':
					case 'override_tax_zone':
					case 'shipping_warehouse_filter':
					case 'shipping_min_price':
					case 'shipping_max_price':
					case 'shipping_virtual_included':
					case 'shipping_price_use_tax':
					case 'shipping_min_quantity':
					case 'shipping_max_quantity':
					case 'shipping_min_weight':
					case 'shipping_weight_unit':
					case 'shipping_max_weight':
					case 'shipping_min_volume':
					case 'shipping_size_unit':
					case 'shipping_max_volume':
					case 'shipping_zip_prefix':
					case 'shipping_min_zip':
					case 'shipping_max_zip':
					case 'shipping_zip_suffix':
					case 'warehouse':
						$userfields[] = $this->db->quoteName('shipping_params');

						if (array_key_exists('shipping_params', $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('shipping_params');
						}

						if (array_key_exists('shipping_params', $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('shipping_params');
						}
						break;
					case 'zone':
						$userfields[] = $this->db->quoteName('shipping_zone_namekey');

						if (array_key_exists('shipping_zone_namekey', $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('shipping_zone_namekey');
						}

						if (array_key_exists('shipping_zone_namekey', $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('shipping_zone_namekey');
						}
						break;
					case 'usergroup_name':
						$userfields[] = $this->db->quoteName('s.shipping_access');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('s.shipping_access');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('s.shipping_access');
						}
						break;
					case 'tax_name':
						$userfields[] = $this->db->quoteName('s.shipping_tax_id');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('s.shipping_tax_id');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('s.shipping_tax_id');
						}
						break;
					case 'currency_code':
						$userfields[] = $this->db->quoteName('s.shipping_currency_id');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('s.shipping_currency_id');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('s.shipping_currency_id');
						}
						break;
					case 'restricted_currency_code':
						$userfields[] = $this->db->quoteName('s.shipping_currency');

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName('s.shipping_currency');
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName('s.shipping_currency');
						}
						break;
					case 'custom':
						break;
					default:
						$userfields[] = $this->db->quoteName($field->field_name);

						if (array_key_exists($field->field_name, $groupbyfields))
						{
							$groupby[] = $this->db->quoteName($field->field_name);
						}

						if (array_key_exists($field->field_name, $sortbyfields))
						{
							$sortby[] = $this->db->quoteName($field->field_name);
						}
						break;
				}
			}

			/**
			 * Export SQL Query
			 * Get all products - including items
			 * as well as products without a price
			 */
			$userfields = array_unique($userfields);
			$query = $this->db->getQuery(true);
			$query->select(implode(",\n", $userfields));
			$query->from($this->db->quoteName('#__hikashop_shipping', 's'));

			// Group the fields
			$groupby = array_unique($groupby);

			if (!empty($groupby))
			{
				$query->group($groupby);
			}

			// Sort set fields
			$sortby = array_unique($sortby);

			if (!empty($sortby))
			{
				$query->order($sortby);
			}

			// Add export limits
			$limits = $this->getExportLimit();

			// Execute the query
			$this->db->setQuery($query, $limits['offset'], $limits['limit']);
			$records = $this->db->getIterator();
			$this->log->add('Export query' . $query->__toString(), false);

			// Check if there are any records
			$logcount = $this->db->getNumRows();

			if ($logcount > 0)
			{
				$query = $this->db->getQuery(true);

				foreach ($records as $record)
				{
					$this->log->incrementLinenumber();

					// Process all the export fields
					foreach ($exportfields as $field)
					{
						// Get the field name
						$fieldname = $field->field_name;

						// Set the field value
						if (isset($record->$fieldname))
						{
							$fieldValue = $record->$fieldname;
						}
						else
						{
							$fieldValue = '';
						}

						// Process the field
						switch ($fieldname)
						{
							case 'shipping_percentage':
							case 'shipping_per_product':
							case 'shipping_price_per_product':
							case 'shipping_override_address':
							case 'shipping_override_address_text':
							case 'override_tax_zone':
							case 'shipping_warehouse_filter':
							case 'shipping_min_price':
							case 'shipping_max_price':
							case 'shipping_virtual_included':
							case 'shipping_price_use_tax':
							case 'shipping_min_quantity':
							case 'shipping_max_quantity':
							case 'shipping_min_weight':
							case 'shipping_weight_unit':
							case 'shipping_max_weight':
							case 'shipping_min_volume':
							case 'shipping_size_unit':
							case 'shipping_max_volume':
							case 'shipping_zip_prefix':
							case 'shipping_min_zip':
							case 'shipping_max_zip':
							case 'shipping_zip_suffix':
								$parameters = unserialize($record->shipping_params);

								if (isset($parameters->$fieldname))
								{
									$fieldValue = $parameters->$fieldname;
								}
								break;
							case 'zone':
								$query->clear()
									->select($this->db->quoteName('zone_name_english'))
									->from($this->db->quoteName('#__hikashop_zone'))
									->where($this->db->quoteName('zone_namekey') . ' = ' . $this->db->quote($record->shipping_zone_namekey));
								$this->db->setQuery($query);

								$fieldValue = $this->db->loadResult();
								break;
							case 'warehouse':
								$parameters = unserialize($record->shipping_params);

								if (isset($parameters->shipping_warehouse_filter))
								{
									$query->clear()
										->select($this->db->quoteName('warehouse_name'))
										->from($this->db->quoteName('#__hikashop_warehouse'))
										->where($this->db->quoteName('warehouse_id') . ' = ' . (int) $parameters->shipping_warehouse_filter);
									$this->db->setQuery($query);

									$fieldValue = $this->db->loadResult();
								}
								break;
							case 'usergroup_name':
								$fieldValue = $record->shipping_access;

								if (strpos($record->shipping_access, ',') !== false)
								{
									$userIds = explode(',', $record->shipping_access);

									if ($userIds)
									{
										$userIds = ArrayHelper::toInteger($userIds);

										$query->clear()
											->select($this->db->quoteName('title'))
											->from($this->db->quoteName('#__usergroups'))
											->where($this->db->quoteName('id') . ' IN (' . implode(',', $userIds) . ')');
										$this->db->setQuery($query);
										$titles = $this->db->loadColumn();
										$this->log->add('Usergroup name');

										$fieldValue = implode('|', $titles);
									}
								}
								break;
							case 'tax_name':
								$fieldValue = $categoryHelper->createCategoryPathById($record->shipping_tax_id);
								break;
							case 'currency_code':
								if ($record->shipping_currency_id)
								{
									$query->clear()
										->select($this->db->quoteName('currency_code'))
										->from($this->db->quoteName('#__hikashop_currency'))
										->where($this->db->quoteName('currency_id') . ' = ' . (int) $record->shipping_currency_id);
									$this->db->setQuery($query);

									$fieldValue = implode('|', $this->db->loadColumn());
								}
								break;
							case 'restricted_currency_code':
								if ($record->shipping_currency)
								{
									$currencyIds = ArrayHelper::toInteger(explode(',', $record->shipping_currency));

									$query->clear()
										->select($this->db->quoteName('currency_code'))
										->from($this->db->quoteName('#__hikashop_currency'))
										->where($this->db->quoteName('currency_id') . ' IN (' . implode(',', $currencyIds) . ')');
									$this->db->setQuery($query);

									$fieldValue = implode('|', $this->db->loadColumn());
								}
								break;
						}

						// Store the field value
						$this->fields->set($field->csvi_templatefield_id, $fieldValue);
					}

					// Output the data
					$this->addExportFields();

					// Output the contents
					$this->writeOutput();
				}
			}
			else
			{
				$this->addExportContent(\JText::_('COM_CSVI_NO_DATA_FOUND'));

				// Output the contents
				$this->writeOutput();
			}
		}
	}
}
