<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace hikashop\com_hikashop\model\import;

defined('_JEXEC') or die;

use Joomla\Utilities\ArrayHelper;

/**
 * Main processor for handling custom fields.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.3.0
 */
class Customfield extends \RantaiImportEngine
{
	/**
	 * The product table
	 *
	 * @var    \HikashopTableCustomfield
	 * @since  6.3.0
	 */
	private $customfieldTable = null;

	/**
	 * The HikaShop helper
	 *
	 * @var    \Com_HikashopHelperCom_Hikashop
	 * @since  6.3.0
	 */
	protected $helper = null;

	/**
	 * The addon config helper
	 *
	 * @var    \Com_HikashopHelperCom_Hikashop_Config
	 * @since  6.3.0
	 */
	protected $helperconfig = null;

	/**
	 * The HikaShop Category helper
	 *
	 * @var    \Com_HikashopHelperCom_Hikashop_Category_Category
	 * @since  6.3.0
	 */
	protected $categoryHelper = null;

	/**
	 * The list of field options
	 *
	 * @var    array
	 * @since  6.6.0
	 */
	private $fieldOptions = array(
			'errormessage',
			'regex',
			'attribute',
			'placeholder',
			'cols',
			'filtering',
			'maxlength',
			'rows',
			'zone_type',
			'pleaseselect',
			'size',
			'format',
			'allow',
			'readonly',
			'limit_to_parent',
			'parent_value',
			'product_id',
			'product_value',
		);

	/**
	 * The list of display options
	 *
	 * @var    array
	 * @since  6.6.0
	 */
	private $displayOptions = array(
		'item' => array(
			'field_item_show_cart',
			'field_item_back_cart_details',
			'field_item_front_cart_details',
			'field_item_checkout',
			'field_item_front_order',
			'field_item_order_edit',
			'field_item_front_product_listing',
			'field_item_product_show',
			'field_item_product_cart',
			'field_item_order_form',
			'field_item_back_invoice',
			'field_item_back_shipping_invoice',
			'field_item_edit_product_order',
			'field_item_mail_order_notif',
			'field_item_mail_status_notif',
			'field_item_mail_order_creation',
			'field_item_mail_admin_notif',
			'field_item_mail_payment_notif',
		),
		'product' => array(
			'field_product_show',
			'field_product_compare',
			'field_product_listing',
			'field_product_front_listing',
			'field_product_form',
			'field_product_back_invoice',
			'field_product_back_shipping_invoice',
			'field_product_order_form',
			'field_product_back_cart_details',
			'field_product_front_cart_details',
			'field_product_mail_order_notif',
			'field_product_mail_status_notif',
			'field_product_mail_order_creation',
			'field_product_mail_admin_notif',
			'field_product_mail_payment_notif',
		),
		'order' => array(
			'field_order_front_order',
			'field_order_invoice',
			'field_order_back_shipping_invoice',
			'field_order_edit',
			'field_order_show',
			'field_order_checkout',
			'field_order_listing',
			'field_order_form',
			'field_order_edit_fields',
			'field_order_mail_order_notif',
			'field_order_mail_status_notif',
			'field_order_mail_order_creation',
			'field_order_mail_admin_notif',
			'field_order_mail_payment_notif',
		)
	);

	/**
	 * Set the state of the record.
	 *
	 * @var    bool
	 * @since  6.6.2
	 */
	private $new = true;

	/**
	 * Load the helpers before we start.
	 *
	 * @return  void
	 *
	 * @since   6.6.0
	 */
	public function onBeforeStart()
	{
		require_once JPATH_PLUGINS . '/csviaddon/hikashop/com_hikashop/helper/com_hikashop_category_category.php';
		$this->categoryHelper = new \Com_HikashopHelperCom_Hikashop_Category_Category($this->db, $this->template, $this->log);
	}

	/**
	 * Here starts the processing.
	 *
	 * @return  bool  Returns true on success | False on failure.
	 *
	 * @since   6.3.0
	 */
	public function getStart()
	{
		// Process data
		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				// Check if the field needs extra treatment
				switch ($name)
				{
					case 'field_frontcomp':
					case 'field_backend':
					case 'field_backend_listing':
					case 'field_item_show_cart':
					case 'field_item_backend_cart_details':
					case 'field_item_checkout':
					case 'field_item_front_order':
					case 'field_item_front_product_listing':
					case 'field_item_product_show':
					case 'field_item_product_cart':
					case 'field_item_order_form':
					case 'field_item_back_invoice':
					case 'field_item_back_shipping_invoice':
					case 'field_item_edit_product_order':
					case 'field_item_mail_order_notif':
					case 'field_item_mail_status_notif':
					case 'field_item_mail_order_creation':
					case 'field_item_mail_admin_notif':
					case 'field_item_mail_payment_notif':
					case 'field_item_order_edit':
					case 'field_item_front_cart_details':
					case 'field_product_show':
					case 'field_product_compare':
					case 'field_product_listing':
					case 'field_product_front_listing':
					case 'field_product_back_invoice':
					case 'field_product_back_shipping_invoice':
					case 'field_product_order_form':
					case 'field_product_back_cart_details':
					case 'field_product_front_cart_details':
					case 'field_product_mail_order_notif':
					case 'field_product_mail_status_notif':
					case 'field_product_mail_order_creation':
					case 'field_product_mail_admin_notif':
					case 'field_product_mail_payment_notif':
					case 'field_front_order':
					case 'field_order_invoice':
					case 'field_order_back_shipping_invoice':
					case 'field_order_edit':
					case 'field_order_show':
					case 'field_order_checkout':
					case 'field_order_listing':
					case 'field_order_form':
					case 'field_order_edit_fields':
					case 'field_order_mail_order_notif':
					case 'field_order_mail_status_notif':
					case 'field_order_mail_order_creation':
					case 'field_order_mail_admin_notif':
					case 'field_order_mail_payment_notif':
					case 'filtering':
					case 'product_value':
						switch (strtolower($value))
						{
							case 'n':
							case 'no':
							case '0':
								$value = 0;
								break;
							default:
								$value = 1;
								break;
						}

						$this->setState($name, $value);
						break;
					case 'field_published':
					case 'field_core':
					case 'field_required':
					case 'field_with_sub_categories':
					case 'readonly':
					case 'translatable':
					case 'pleaseselect':
						switch (strtolower($value))
						{
							case 'y':
							case 'yes':
							case '1':
								$value = 1;
								break;
							default:
								$value = 0;
								break;
						}

						$this->setState($name, $value);
						break;
					default:
						$this->setState($name, $value);
						break;
				}
			}
		}

		if ($this->getState('field_namekey', false))
		{
			// Load the current discount data
			$this->loaded = true;

			// See if we have a field_id
			if (!$this->getState('field_id', false))
			{
				// Find the custom field ID
				$query = $this->db->getQuery(true)
					->select($this->db->quoteName('field_id'))
					->from($this->db->quoteName('#__hikashop_field'))
					->where($this->db->quoteName('field_namekey') . ' = ' . $this->db->quote($this->getState('field_namekey')));
				$this->db->setQuery($query);
				$this->log->add('Find the field ID', false);
				$this->setState('field_id', $this->db->loadResult());
			}

			if ($fieldId = $this->getState('field_id', false))
			{
				$this->new = false;
			}

			if ($this->customfieldTable->load($fieldId))
			{
				if (!$this->template->get('overwrite_existing_data'))
				{
					$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_CONTENT', $this->getState('field_namekey', '')));
					$this->loaded = false;
				}
			}
		}
		else
		{
			$this->loaded = false;

			$this->log->addStats('skipped', \JText::_('COM_CSVI_NO_COLUMN_NAME_SET'));
		}

		if (!$this->getState('field_table', false))
		{
			$this->loaded = false;

			$this->log->addStats('skipped', \JText::_('COM_CSVI_NO_COLUMN_TABLE_SET'));
		}

		return true;
	}

	/**
	 * Process each record and store it in the database.
	 *
	 * @return  bool  Returns true if all is OK | Returns false if no product SKU or product ID can be found.
	 *
	 * @since   3.0
	 */
	public function getProcessRecord()
	{
		if ($this->loaded)
		{
			// Get the needed data
			$field_id = $this->getState('field_id', false);
			$field_delete = $this->getState('field_delete', 'N');

			// User wants to delete the product
			if ($field_id && $field_delete == 'Y')
			{
				$this->deleteField();
			}
			elseif (!$field_id && $field_delete == 'Y')
			{
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_NO_CUSTOMFIELD_ID_NO_DELETE', $this->getState('field_namekey')));
			}
			elseif (!$field_id && $this->template->get('ignore_non_exist'))
			{
				// Do nothing for new products when user chooses to ignore new products
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_IGNORE_NEW', $this->getState('field_namekey')));
			}
			else
			{
				// Process the field options
				$this->processFieldOptions();

				// Process the display options
				$this->processDisplayOptions();

				// Handle the access level(s)
				$this->processAccessLevel();

				// Handle the categories
				$this->processCategories();

				// Handle the field values
				$this->processFieldValues();

				// Handle the database column creation
				$this->processDatabaseTable();

				$this->customfieldTable->bind($this->state);

				try
				{
					$this->customfieldTable->check();
					$this->customfieldTable->store();
					$this->log->add('Custom fields added', false);
				}
				catch (\Exception $e)
				{
					$this->log->add('Cannot add custom fields. Error: ' . $e->getMessage(), false);

					return false;
				}
			}

			return true;
		}
		else
		{
			return false;
		}
	}

	/**
	 * Load the discount related tables.
	 *
	 * @return  void.
	 *
	 * @since   6.3.0
	 */
	public function loadTables()
	{
		// Load the main tables
		$this->customfieldTable = $this->getTable('Customfield');
	}

	/**
	 * Cleaning the product related tables.
	 *
	 * @return  void.
	 *
	 * @since   6.3.0
	 */
	public function clearTables()
	{
		// Clean the main tables
		$this->customfieldTable->reset();
	}

	/**
	 * Delete a discount.
	 *
	 * @return  void.
	 *
	 * @since   6.3.0
	 */
	private function deleteField()
	{
		if (!$this->customfieldTable->delete($this->getState('field_id')))
		{
			$this->log->addStats('incorrect', 'COM_CSVI_CUSTOMFIELD_NOT_DELETED');
		}

		// Remove the database field as well
		$fieldTable   = $this->getState('field_table', false);
		$fieldNameKey = $this->getState('field_namekey', false);

		if ($fieldTable && $fieldNameKey)
		{
			$tables = array($fieldTable);

			if ($fieldTable === 'item')
			{
				$tables = array('cart_product', 'order_product');
			}

			foreach ($tables as $table_name)
			{
				$columns = $this->db->getTableColumns('#__hikashop_' . $table_name);

				if (isset($columns[$fieldNameKey]))
				{
					$query = 'ALTER TABLE ' . $this->db->quoteName('#__hikashop_' . $table_name) . ' DROP ' . $this->db->quoteName($fieldNameKey);
					$this->db->setQuery($query)->execute();
					$this->log->add('Remove field ' . $fieldNameKey . ' from table ' . $fieldTable);
				}
			}
		}
	}

	/**
	 * Process the field options options.
	 *
	 * @return  void
	 *
	 * @since   6.6.0
	 */
	private function processFieldOptions()
	{
		// Load the existing options
		$fieldValues = unserialize($this->customfieldTable->get('field_options'));

		if (false === $fieldValues)
		{
			$fieldValues = array();
		}

		foreach ($this->fieldOptions as $option)
		{
			// Check if the key exists
			if (!array_key_exists($option, $fieldValues))
			{
				$fieldValues[$option] = '';
			}

			switch ($option)
			{
				case 'allow':
				case 'limit_to_parent':
					$value = $this->getState($option);

					// Check if we have a value, then user is importing data
					if ($value)
					{
						if ($value === 'all')
						{
							$value = '';
						}

						$this->setState($option, $value);
					}
					break;
				default:
					// Set the value
					$fieldValues[$option] = $this->getState($option, $fieldValues[$option]);
					break;
			}
		}

		// Special fields treatment
		if ($productCode = $this->getState('product_code', false))
		{
			$fieldValues['product_id'] = $this->helper->getProductId($productCode);
		}

		// Store the new values
		$this->setState('field_options', serialize($fieldValues));
	}

	/**
	 * Process the field options options. The field_table is required to know which options to store.
	 *
	 * @return  void
	 *
	 * @since   6.6.0
	 */
	private function processDisplayOptions()
	{
		// Get the table
		$table = $this->getState('field_table');

		$this->log->add('Process display options for table ' . $table, false);

		if (array_key_exists($table, $this->displayOptions))
		{
			// Load the existing options
			$displayData = explode(';', $this->customfieldTable->get('field_display', ''));

			// Get the defaults
			$fieldValues = array();
			$found       = false;

			foreach ($displayData as $dataValue)
			{
				if (false !== strpos($dataValue, '='))
				{
					list($displayName, $displayValue) = explode('=', $dataValue);

					$search = 'field_' . $table . '_';
					$displayName = str_replace($search, '', $displayName);

					$fieldValues[$displayName] = $displayValue;
				}
			}

			foreach ($this->displayOptions[$table] as $option)
			{
				$value = $this->getState($option, false);

				if (false !== $value)
				{
					$search = 'field_' . $table . '_';
					$option = str_replace($search, '', $option);

					// Set the value
					$fieldValues[$option] = $value;
					$found = true;
				}
			}

			if ($found)
			{
				$fieldDisplay = ';';

				foreach ($fieldValues as $key => $value)
				{
					$fieldDisplay .= $key . '=' . $value . ';';
				}

				// Store the new values
				$this->setState('field_display', $fieldDisplay);
			}
		}
	}

	/**
	 * Process the categories.
	 *
	 * @return  bool  True on sucess | False on failure.
	 *
	 * @since   6.3.0
	 */
	private function processCategories()
	{
		if ($categoryPaths = $this->getState('category_path', false))
		{
			if ($categoryPaths !== 'all')
			{
				$categories = explode('|', $categoryPaths);
				$categoryIds = array();

				foreach ($categories as $order => $path)
				{
					$this->log->add('Process path: ' . $path, false);

					$categoryIds[] = $this->categoryHelper->getCategoryIdByPath($path);
				}

				$categoryIds = ArrayHelper::toInteger($categoryIds);

				// Add the first and last entry
				array_unshift($categoryIds, '');
				$categoryIds[] = '';


				if ($categoryIds)
				{
					$categoryPaths = implode(',', $categoryIds);
				}
			}

			$this->setState('field_categories', $categoryPaths);
		}
	}

	/**
	 * Process the shopper groups.
	 *
	 * @return  bool  True on OK | False on failure.
	 *
	 * @since   4.5.2
	 */
	private function processAccessLevel()
	{
		$accessField = $this->getState('usergroup_name', false);

		if ($accessField && $accessField != 'all')
		{
			// Get the shopper group names
			$names = explode('|', $accessField);
			$ids = array();

			foreach ($names as $name)
			{
				$ids[] = $this->helper->getAccessLevelId($name);
			}

			// Clear the array
			$ids = ArrayHelper::toInteger($ids);

			// Set the access level ID for other updates
			$this->setState('field_access', implode(',', $ids));
		}

		return true;
	}

	/**
	 * Process the field values.
	 *
	 * @return  void
	 *
	 * @since   6.6.0
	 */
	private function processFieldValues()
	{
		$this->setState('field_value', str_replace('|', "\r\n", $this->getState('field_value', '')));
	}

	/**
	 * Process the database table creation.
	 *
	 * @return  void
	 *
	 * @since   6.6.2
	 */
	private function processDatabaseTable()
	{
		if ($this->new)
		{
			$fieldTable   = $this->getState('field_table', false);
			$fieldNameKey = $this->getState('field_namekey', false);

			if ($fieldTable && $fieldNameKey)
			{
				$tables = array($fieldTable);

				if ($fieldTable === 'item')
				{
					$tables = array('cart_product', 'order_product');
				}

				foreach ($tables as $table_name)
				{
					$columns = $this->db->getTableColumns('#__hikashop_' . $table_name);

					if (!isset($columns[$fieldNameKey]))
					{
						$query = 'ALTER TABLE ' . $this->db->quoteName('#__hikashop_' . $table_name) . ' ADD ' . $this->db->quoteName($fieldNameKey) . ' TEXT NULL';
						$this->db->setQuery($query)->execute();

						$query = $this->db->getQuery(true)
							->insert($this->db->quoteName('#__csvi_availablefields'))
							->columns($this->db->quoteName(array('csvi_name', 'component_name', 'component_table', 'component', 'action')));

						$query->values(
							$this->db->quote($fieldNameKey) . ', ' .
							$this->db->quote($fieldNameKey) . ', ' .
							$this->db->quote('hikashop_product') . ', ' .
							$this->db->quote('com_hikashop') . ', ' .
							$this->db->quote('import')
						);

						$query->values(
							$this->db->quote($fieldNameKey) . ', ' .
							$this->db->quote($fieldNameKey) . ', ' .
							$this->db->quote('hikashop_product') . ', ' .
							$this->db->quote('com_hikashop') . ', ' .
							$this->db->quote('export')
						);

						$this->db->setQuery($query)->execute();
					}
				}
			}
		}
	}
}
