<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace hikashop\com_hikashop\model\import;

defined('_JEXEC') or die;

/**
 * Order product import.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       7.0
 */
class Orderproduct extends \RantaiImportEngine
{
	/**
	 * Order product table.
	 *
	 * @var    \HikashopTableOrderProduct
	 * @since  7.0
	 */
	private $orderProductTable = null;

	/**
	 * The Com_HikashopHelperCom_Hikashop
	 *
	 * @var    \Com_HikashopHelperCom_Hikashop
	 * @since  7.0
	 */
	protected $helper = null;

	/**
	 * Start the order product import process.
	 *
	 * @return  bool  Always returns true.
	 *
	 * @since   7.0
	 */
	public function getStart()
	{
		// Process data
		foreach ($this->fields->getData() as $fields)
		{
			foreach ($fields as $name => $details)
			{
				$value = $details->value;

				switch ($name)
				{
					case 'product_price':
						$this->setState('order_product_price', $this->cleanPrice($value));
						break;
					case 'product_code':
						$this->setState('order_product_code', $value);
						break;
					case 'product_name':
						$this->setState('order_product_name', $value);
						break;
					default:
						$this->setState($name, $value);
						break;
				}
			}
		}

		// Reset loaded state
		$this->loaded = true;

		// Check for product id
		if (!$this->getState('product_id', false) && $this->getState('order_product_code', false))
		{
			$this->setState('product_id', $this->helper->getProductId($this->getState('order_product_code', false)));

			if (!$this->getState('product_id', false))
			{
				$this->log->addStats('incorrect', \JText::sprintf('COM_CSVI_NO_PRODUCT_ID_FOUND', $this->getState('order_product_code')));
				$this->loaded = false;
			}
		}
		elseif ($this->getState('product_id', false) && !$this->getState('order_product_code', false))
		{
			$query = $this->db->getQuery(true)
				->select($this->db->quoteName('product_code'))
				->from($this->db->quoteName('#__hikashop_product'))
				->where($this->db->quoteName('product_id') . ' = ' . (int) $this->getState('product_id'));
			$this->db->setQuery($query);
			$this->setState('order_product_code', $this->db->loadResult());
		}
		elseif (!$this->getState('product_id', false) && !$this->getState('order_product_code', false))
		{
			$this->log->addStats('incorrect', \JText::_('COM_CSVI_NO_PRODUCT_ID_OR_CODE'));
			$this->loaded = false;
		}

		// Required fields are order_id, order_product_code or product_id
		if (!$this->getState('order_id', false)
			&& (!$this->getState('order_product_code', false) || !$this->getState('product_id', false)))
		{
			$this->log->addStats('incorrect', \JText::_('COM_CSVI_MISSING_REQUIRED_FIELDS_ORDER'));
			$this->loaded = false;
		}

		// Check if there is order number given and no order id
		if (!$this->getState('order_id', false) && $this->getState('order_number', false))
		{
			$orderId = $this->helper->getOrderId($this->getState('order_number', false));

			if (!$orderId)
			{
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_ORDER_ID_NOT_FOUND', $this->getState('order_number', false)));
				$this->loaded = false;
			}

			$this->setState('order_id', $orderId);
		}

		// Bind the values
		$this->orderProductTable->bind($this->state);

		if ($this->orderProductTable->check())
		{
			$this->setState('order_product_id', $this->orderProductTable->get('order_product_id'));

			// Check if we have an existing item
			if ($this->getState('order_product_id', 0) && !$this->template->get('overwrite_existing_data', true))
			{
				$this->log->add("The product code " . $this->getState('order_product_code') . " already exists for order ID
				" . $this->getState('order_id') . " and the option Overwrite data is set to No");
				$this->log->addStats('skipped',
					\JText::sprintf('COM_CSVI_DATA_EXISTS_ORDERITEM', $this->getState('order_product_code'), $this->getState('order_id')
					)
				);

				$this->loaded = false;
			}
		}

		return true;
	}

	/**
	 * Process a record.
	 *
	 * @return  bool  Returns true if all is OK | Returns false otherwise.
	 *
	 * @since   7.0
	 */
	public function getProcessRecord()
	{
		if ($this->loaded)
		{
			if (!$this->getState('order_product_id', false) && $this->template->get('ignore_non_exist'))
			{
				// Do nothing for new rules when user chooses to ignore new rules
				$this->log->addStats('skipped', \JText::sprintf('COM_CSVI_DATA_EXISTS_IGNORE_NEW', $this->getState('order_product_code')));

				return false;
			}

			$this->orderProductTable->load();

			// Check if we have a product name
			if (!$this->orderProductTable->get('order_product_name', false))
			{
				$query = $this->db->getQuery(true)
					->select($this->db->quoteName('product_name'))
					->from($this->db->quoteName('#__hikashop_product'))
					->where($this->db->quoteName('product_id') . ' = ' . (int) $this->getState('product_id'));
				$this->db->setQuery($query);
				$this->setState('order_product_name', $this->db->loadResult());
			}

			// Handle the shipping method
			if ($this->getState('order_product_shipping_method', false))
			{
				$query = $this->db->getQuery(true);
				$query->select($this->db->quoteName('shipping_id'));
				$query->from($this->db->quoteName('#__hikashop_shipping'));
				$query->where(
					$this->db->quoteName('shipping_name') . ' =
			' . $this->db->quote($this->getState('order_product_shipping_method'))
				);

				$this->db->setQuery($query);
				$this->setState('order_product_shipping_id', $this->db->loadResult());
				$this->log->add('Found order product shipping method');
			}

			// Check and process product tax for an order
			if (!$this->getState('order_product_tax_info', false))
			{
				$productTaxInfo = json_decode($this->orderProductTable->order_product_tax_info);

				if (!is_object($productTaxInfo))
				{
					$productTaxInfo = new \stdClass;
				}

				$productTaxInfo->tax_namekey = $this->getState('tax_namekey', '');

				if (!$this->getState('tax_namekey', false))
				{
					$productTaxInfo->tax_namekey = $this->getState('tax_namekey', false);
				}

				$productTaxInfo->tax_amount = $this->getState('tax_amount', '');

				if (!$this->getState('tax_amount', false))
				{
					$productTaxInfo->tax_amount = $this->getState('tax_amount', false);
				}

				if (!$this->getState('order_product_tax', false) && $this->getState('tax_amount', false))
				{
					$this->setState('order_product_tax', $this->getState('tax_amount', false));
				}

				if ($productTaxInfo)
				{
					$this->setState('order_product_tax_info', serialize(json_encode($productTaxInfo)));
				}
			}

			// Bind the data
			$this->orderProductTable->bind($this->state);

			try
			{
				$this->orderProductTable->store();
				$this->log->add('Order product saved', false);
			}
			catch (\Exception $e)
			{
				$this->log->add('Order product not saved: ' . $e->getMessage(), false);
				$this->log->addStats('incorrect', \JText::sprintf('COM_CSVI_ORDER_ITEM_NOT_ADDED', $e->getMessage()));
			}

			return true;
		}

		return false;
	}

	/**
	 * Load the necessary tables.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	public function loadTables()
	{
		$this->orderProductTable = $this->getTable('OrderProduct');
	}

	/**
	 * Clear the loaded tables.
	 *
	 * @return  void.
	 *
	 * @since   7.0
	 */
	public function clearTables()
	{
		$this->orderProductTable->reset();
	}
}

