<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

/**
 * HikaShop Address table.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.0
 */
class HikashopTableAddress extends CsviTableDefault
{
	/**
	 * Table constructor.
	 *
	 * @param   string     $table   Name of the database table to model.
	 * @param   string     $key     Name of the primary key field in the table.
	 * @param   JDatabase  &$db     Database driver
	 * @param   array      $config  The configuration parameters array
	 *
	 * @since   4.0
	 */
	public function __construct($table, $key, &$db, $config = array())
	{
		parent::__construct('#__hikashop_address', 'address_id', $db, $config);
	}

	/**
	 * Check if there is already an address for this customer.
	 *
	 * @return  bool  Always returns true.
	 *
	 * @since   1.0
	 *
	 * @throws  InvalidArgumentException
	 */
	public function check()
	{
		if (!$this->get('address_user_id', false))
		{
			throw new InvalidArgumentException(JText::_('COM_CSVI_MISSING_USER_ID'));
		}

		if (!$this->get('address_id', false))
		{
			// Check for an existing address
			$query = $this->db->getQuery(true)
				->select($this->db->quoteName($this->_tbl_key))
				->from($this->db->quoteName($this->_tbl))
				->where($this->db->quoteName('address_user_id') . ' = ' . (int) $this->get('address_user_id'))
				->where($this->db->quoteName('address_default') . ' = 1');
			$this->db->setQuery($query);

			$this->set('address_id', $this->db->loadResult());
		}

		return true;
	}

	/**
	 * Reset the primary key.
	 *
	 * @return  bool  Returns true on success.
	 *
	 * @since   6.0
	 */
	public function reset()
	{
		parent::reset();

		// Empty the primary key
		$this->set('address_id', null);

		return true;
	}
}
