<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

/**
 * HikaShop Category table.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.0
 */
class HikashopTableCategory extends CsviTableDefault
{
	/**
	 * Table constructor.
	 *
	 * @param   string     $table   Name of the database table to model.
	 * @param   string     $key     Name of the primary key field in the table.
	 * @param   JDatabase  &$db     Database driver
	 * @param   array      $config  The configuration parameters array
	 *
	 * @since   4.0
	 */
	public function __construct($table, $key, &$db, $config = array())
	{
		parent::__construct('#__hikashop_category', 'category_id', $db, $config);
	}

	/**
	 * Do a sanity check before storing the data.
	 *
	 * @return  bool  True on OK | False on NOK.
	 *
	 * @since   6.0
	 */
	public function check()
	{
		// Load the transliterate helper
		$translit = new CsviHelperTranslit($this->template);

		// Check the alias
		if (empty($this->category_alias))
		{
			$this->set('category_alias', $this->get('category_name'));
		}

		$this->category_alias = $translit->stringURLSafe($this->category_alias);

		return true;
	}

	/**
	 * Reset the primary key.
	 *
	 * @return  bool  Returns true on success.
	 *
	 * @since   6.0
	 */
	public function reset()
	{
		parent::reset();

		// Empty the primary key
		$this->set('category_id', null);

		return true;
	}
}
