<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

/**
 * HikaShop User table.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.0
 */
class HikashopTableHsuser extends CsviTableDefault
{
	/**
	 * Table constructor.
	 *
	 * @param   string     $table   Name of the database table to model.
	 * @param   string     $key     Name of the primary key field in the table.
	 * @param   JDatabase  &$db     Database driver
	 * @param   array      $config  The configuration parameters array
	 *
	 * @since   4.0
	 */
	public function __construct($table, $key, &$db, $config = array())
	{
		parent::__construct('#__hikashop_user', 'user_id', $db, $config);
	}

	/**
	 * Check the data before insert.
	 *
	 * @return  bool  Always returns true.
	 *
	 * @since   1.0
	 */
	public function check()
	{
		if (!$this->get('user_id', false))
		{
			// Check if there is already an entry
			$pk = 'user_email';

			if ($this->get('user_cms_id', false))
			{
				$pk = 'user_cms_id';
			}

			$query = $this->db->getQuery(true)
				->select($this->db->quoteName($this->_tbl_key))
				->from($this->db->quoteName($this->_tbl))
				->where($this->db->quoteName($pk) . ' = ' . $this->db->quote($this->$pk));
			$this->db->setQuery($query);

			$this->log->add('Check if HikaShop user exists');
			$this->set('user_id', $this->db->loadResult());
		}

		return true;
	}

	/**
	 * Reset the primary key.
	 *
	 * @return  bool  Returns true on success.
	 *
	 * @since   6.0
	 */
	public function reset()
	{
		parent::reset();

		// Empty the primary key
		$this->set('user_id', null);

		return true;
	}
}
