<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

/**
 * HikaShop Order Product table.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       7.0
 */
class HikashopTableOrderProduct extends CsviTableDefault
{
	/**
	 * Table constructor.
	 *
	 * @param   string     $table   Name of the database table to model.
	 * @param   string     $key     Name of the primary key field in the table.
	 * @param   JDatabase  &$db     Database driver
	 * @param   array      $config  The configuration parameters array
	 *
	 * @since   4.0
	 */
	public function __construct($table, $key, &$db, $config = array())
	{
		parent::__construct('#__hikashop_order_product', 'order_product_id', $db, $config);
	}

	/**
	 * Check if a given order product already exists.
	 *
	 * @return  boolean  True if the instance is sane and able to be stored in the database.
	 *
	 * @since   7.0
	 */
	public function check()
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName($this->_tbl_key))
			->from($this->db->quoteName($this->_tbl))
			->where($this->db->quoteName('order_id') . ' = ' . (int) $this->get('order_id'))
			->where($this->db->quoteName('order_product_code') . ' = ' . $this->db->quote($this->get('order_product_code')));
		$this->db->setQuery($query);
		$result = $this->db->loadResult();

		if ($result)
		{
			$this->set('order_product_id', $result);

			return true;
		}

		return false;
	}

	/**
	 * Reset the primary key.
	 *
	 * @return  bool  Returns true on success.
	 *
	 * @since   6.0
	 */
	public function reset()
	{
		parent::reset();

		// Empty the primary key
		$this->set('order_product_id', null);

		return true;
	}
}
