<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

/**
 * HikaShop Product related table.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.0
 */
class HikashopTableProductRelated extends CsviTableDefault
{
	/**
	 * Table constructor.
	 *
	 * @param   string     $table   Name of the database table to model.
	 * @param   string     $key     Name of the primary key field in the table.
	 * @param   JDatabase  &$db     Database driver
	 * @param   array      $config  The configuration parameters array
	 *
	 * @since   4.0
	 */
	public function __construct($table, $key, &$db, $config = array())
	{
		// We need a compound key
		$keys = array(
			'product_id',
			'product_related_id',
			'product_related_type',
		);

		parent::__construct('#__hikashop_product_related', $keys, $db, $config);
	}

	/**
	 * Delete all related products for given product ID.
	 *
	 * @param   int  $product_id  The product to delete related products for.
	 * @param   int  $relation    The type of relation.
	 *
	 * @return  bool  True if deleted | False if not deleted.
	 *
	 * @since   4.0
	 */
	public function deleteRelated($product_id, $relation)
	{
		$query = $this->db->getQuery(true)
			->delete($this->db->quoteName($this->_tbl))
			->where($this->db->quoteName('product_id') . ' = ' . (int) $product_id)
			->where($this->db->quoteName('product_related_type') . ' = ' . $this->db->quote($relation));
		$this->db->setQuery($query);

		return $this->db->execute();
	}

	/**
	 * Reset the primary key.
	 *
	 * @return  boolean  Always returns true.
	 *
	 * @since   6.0
	 */
	public function reset()
	{
		parent::reset();

		// Empty the primary keys
		foreach ($this->_tbl_keys as $key)
		{
			$this->$key = null;
		}

		return true;
	}
}
