<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

/**
 * Joomla User table.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.0
 */
class HikashopTableuser extends CsviTableDefault
{
	/**
	 * Table constructor.
	 *
	 * @param   string     $table   Name of the database table to model.
	 * @param   string     $key     Name of the primary key field in the table.
	 * @param   JDatabase  &$db     Database driver
	 * @param   array      $config  The configuration parameters array
	 *
	 * @since   6.0
	 */
	public function __construct($table, $key, &$db, $config = array())
	{
		parent::__construct('#__users', 'id', $db, $config);
	}

	/**
	 * Check if username already exists
	 *
	 * @return  bool  True on OK | False on failure.
	 *
	 * @since   6.3.0
	 */
	public function checkUsername()
	{
		// Check if the username is unique
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('id'))
			->from($this->db->quoteName('#__users'))
			->where($this->db->quoteName('username') . ' = ' . $this->db->quote($this->username))
			->where($this->db->quoteName('id') . ' != ' . $this->db->quote($this->id));

		$this->db->setQuery($query);
		$result = $this->db->loadResult();
		$this->log->add('Check if username already exists');

		// If the username exists, we can't continue
		if ($result !== null)
		{
			$this->log->add('User with username ' . $this->username . ' already exists');

			return false;
		}

		return true;
	}

	/**
	 * Reset the primary key.
	 *
	 * @return  bool  Returns true on success.
	 *
	 * @since   6.0
	 */
	public function reset()
	{
		parent::reset();

		// Empty the primary key
		$this->set('id', null);

		return true;
	}
}
