<?php
/**
 * @package     CSVI
 * @subpackage  HikaShop
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

/**
 * HikaShop addon installer.
 *
 * @package     CSVI
 * @subpackage  HikaShop
 * @since       6.0
 */
class PlgcsviaddonhikashopInstallerScript
{
	/**
	 * Actions to perform before installation.
	 *
	 * @param   string  $route   The type of installation being run.
	 * @param   object  $parent  The parent object.
	 *
	 * @return  bool  True on success | False on failure.
	 *
	 * @since   6.0
	 */
	public function preflight($route, $parent)
	{
		if ($route == 'install')
		{
			// Check if RO CSVI is installed
			if (!file_exists(JPATH_ADMINISTRATOR . '/components/com_csvi/'))
			{
				JFactory::getApplication()->enqueueMessage(JText::_('PLG_CSVIADDON_CSVI_NOT_INSTALLED'), 'error');

				return false;
			}
		}

		return true;
	}

	/**
	 * Actions to perform after installation.
	 *
	 * @param   object  $parent  The parent object.
	 *
	 * @return  bool  True on success | False on failure.
	 *
	 * @since   6.0
	 */
	public function postflight($parent)
	{
		// Load the application
		$app = JFactory::getApplication();
		$db = JFactory::getDbo();

		// Clear the cache
		$cache = JFactory::getCache('com_csvi', '');
		$cache->clean('com_csvi');

		try
		{
			// Load the tasks
			$queries = $db->splitSql(file_get_contents(JPATH_PLUGINS . '/csviaddon/hikashop/com_hikashop/install/tasks.sql'));

			foreach ($queries as $query)
			{
				$query = trim($query);

				if (!empty($query))
				{
					$db->setQuery($query)->execute();
				}
			}

			// Enable the plugin
			$query = $db->getQuery(true)
				->update($db->quoteName('#__extensions'))
				->set($db->quoteName('enabled') . ' =  1')
				->where($db->quoteName('type') . ' = ' . $db->quote('plugin'))
				->where($db->quoteName('element') . ' = ' . $db->quote('hikashop'))
				->where($db->quoteName('folder') . ' = ' . $db->quote('csviaddon'));

			$db->setQuery($query)->execute();
			$app->enqueueMessage(JText::_('PLG_CSVIADDON_PLUGIN_ENABLED'));
		}
		catch (Exception $e)
		{
			$app->enqueueMessage($e->getMessage());

			return false;
		}

		return true;
	}
}
