<?php
/**
 * @package     CSVI
 * @subpackage  K2
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

/**
 * K2 Config class.
 *
 * @package     CSVI
 * @subpackage  K2
 * @since       6.0
 */
class Com_K2HelperCom_K2_Config
{
	/**
	 * Holds the K2 configuration
	 *
	 * @var    JRegistry
	 * @since  4.0
	 */
	private $cfg = array();

	/**
	 * Constructor.
	 *
	 * @since  4.0
	 */
	public function __construct()
	{
		$this->parse();
	}

	/**
	 * Finds a given K2 setting.
	 *
	 * @param   string  $setting  The config value to find.
	 *
	 * @return  mixed  Value if found | False if not found.
	 *
	 * @since   4.0
	 */
	public function get($setting)
	{
		return $this->cfg->get($setting, false);
	}

	/**
	 * Parse the K2 configuration.
	 *
	 * @return  void.
	 *
	 * @since   4.0
	 */
	private function parse()
	{
		$db = JFactory::getDbo();
		$query = $db->getQuery(true)
			->select($db->quoteName('params'))
			->from($db->quoteName('#__extensions'))
			->where($db->quoteName('element') . ' = ' . $db->quote('com_k2'))
			->where($db->quoteName('type') . ' = ' . $db->quote('component'));
		$db->setQuery($query);
		$params = $db->loadResult();

		$this->cfg = new JRegistry;
		$this->cfg->loadString($params);
	}
}
