<?php
/**
 * @package     CSVI
 * @subpackage  K2
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

/**
 * Category table.
 *
 * @package     CSVI
 * @subpackage  K2
 * @since       6.0
 */
class K2TableCategory extends CsviTableDefault
{
	/**
	 * Table constructor.
	 *
	 * @param   string     $table   Name of the database table to model.
	 * @param   string     $key     Name of the primary key field in the table.
	 * @param   JDatabase  &$db     Database driver
	 * @param   array      $config  The configuration parameters array
	 *
	 * @since   4.0
	 */
	public function __construct($table, $key, &$db, $config = array())
	{
		parent::__construct('#__k2_categories', 'id', $db, $config);
	}

	/**
	 * The event which runs after deleting a record
	 *
	 * @param   integer  $oid  The PK value of the record which was deleted
	 *
	 * @return  boolean  True to allow the deletion without errors
	 */
	protected function onAfterDelete($oid)
	{
		// Reset the products that link to this category
		$query = $this->db->getQuery(true)
			->update($this->db->quoteName('#__k2_items'))
			->set($this->db->quoteName('catid') . ' = 0')
			->where($this->db->quoteName('catid') . ' = ' . (int) $this->id);
		$this->db->setQuery($query);
		$this->log->add('COM_CSVI_DEBUG_DELETE_ITEM_CATEGORY_XREF', true);

		return $this->db->execute();
	}

	/**
	 * Reset the primary key.
	 *
	 * @return  boolean  Always returns true.
	 *
	 * @since   6.0
	 */
	public function reset()
	{
		parent::reset();

		// Empty the primary key
		$this->id = null;

		return true;
	}
}
