<?php
/**
 * @package     CSVI
 * @subpackage  Productbuilder
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

/**
 * Product Builder helper class.
 *
 * @package     CSVI
 * @subpackage  Productbuilder
 * @since       6.6.0
 */
class Com_ProductbuilderHelperCom_Productbuilder
{
	/**
	 * Template helper
	 *
	 * @var    CsviHelperTemplate
	 * @since  6.6.0
	 */
	protected $template;

	/**
	 * Logger helper
	 *
	 * @var    CsviHelperLog
	 * @since  6.6.0
	 */
	protected $log;

	/**
	 * Fields helper
	 *
	 * @var    CsviHelperFields
	 * @since  6.6.0
	 */
	protected $fields;

	/**
	 * Database connector
	 *
	 * @var    JDatabaseDriver
	 * @since  6.6.0
	 */
	protected $db;

	/**
	 * Constructor.
	 *
	 * @param   CsviHelperTemplate  $template  An instance of CsviHelperTemplate.
	 * @param   CsviHelperLog       $log       An instance of CsviHelperLog.
	 * @param   CsviHelperFields    $fields    An instance of CsviHelperFields.
	 * @param   JDatabaseDriver     $db        Database connector.
	 *
	 * @since   4.0
	 */
	public function __construct(CsviHelperTemplate $template, CsviHelperLog $log, CsviHelperFields $fields, JDatabaseDriver $db)
	{
		$this->template = $template;
		$this->log = $log;
		$this->fields = $fields;
		$this->db = $db;
	}

	/**
	 * Load the product ID.
	 *
	 * @param   string  $productSku  The SKU of the product
	 *
	 * @return  int  The ID of the product.
	 *
	 * @since   6.6.0
	 *
	 * @throws  RuntimeException
	 */
	public function loadProductId($productSku)
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('id'))
			->from($this->db->quoteName('#__pb_products'))
			->where($this->db->quoteName('sku') . ' = ' . $this->db->quote($productSku));
		$this->db->setQuery($query);

		return $this->db->loadResult();
	}
}
