<?php
/**
 * @package     CSVI
 * @subpackage  Productbuilder
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace productbuilder\com_productbuilder\model\export;

defined('_JEXEC') or die;

/**
 * Export Product Builder products.
 *
 * @package     CSVI
 * @subpackage  Productbuilder
 * @since       6.6.0
 */
class Product extends \CsviModelExports
{
	/**
	 * The addon helper
	 *
	 * @var    \Com_ProductbuilderHelperCom_Productbuilder
	 * @since  6.6.0
	 */
	protected $helper;

	/**
	 * Export the data.
	 *
	 * @return  void.
	 *
	 * @since   6.6.0
	 *
	 * @throws  \CsviException
	 * @throws  \RuntimeException
	 */
	protected function exportBody()
	{
		if (parent::exportBody())
		{
			// Get some basic data
			$userFields = array();
			$exportFields = $this->fields->getFields();

			// Group by fields
			$groupFields = json_decode($this->template->get('groupbyfields', '', 'string'));
			$groupBy = array();
			$groupByFields = array();

			if (isset($groupFields->name))
			{
				$groupByFields = array_flip($groupFields->name);
			}

			// Sort selected fields
			$sortFields = json_decode($this->template->get('sortfields', '', 'string'));
			$sortBy = array();
			$sortByFields = array();

			if (isset($sortFields->name))
			{
				$sortByFields = array_flip($sortFields->name);
			}

			foreach ($exportFields as $field)
			{
				switch ($field->field_name)
				{
					case 'custom':
						break;
					default:
						$userFields[] = $this->db->quoteName($field->field_name);

						if (array_key_exists($field->field_name, $groupByFields))
						{
							$groupBy[] = $this->db->quoteName($field->field_name);
						}

						if (array_key_exists($field->field_name, $sortByFields))
						{
							$sortBy[] = $this->db->quoteName($field->field_name);
						}
						break;
				}
			}

			// Build the query
			$userFields = array_unique($userFields);
			$query = $this->db->getQuery(true)
				->select(implode(",\n", $userFields))
				->from($this->db->quoteName('#__pb_products', 'p'));

			// Filter by published state
			$publish_state = $this->template->get('publish_state');

			if ('' !== $publish_state && ($publish_state === 1 || $publish_state === 0))
			{
				$query->where($this->db->quoteName('p.published') . ' = ' . (int) $publish_state);
			}

			// Group the fields
			$groupBy = array_unique($groupBy);

			if (0 !== count($groupBy))
			{
				$query->group($groupBy);
			}

			// Sort set fields
			$sortBy = array_unique($sortBy);

			if (0 !== count($sortBy))
			{
				$query->order($sortBy);
			}

			// Add export limits
			$limits = $this->getExportLimit();

			// Execute the query
			$this->db->setQuery($query, $limits['offset'], $limits['limit']);
			$records = $this->db->getIterator();
			$this->log->add('Export query' . $query->__toString(), false);

			// Check if there are any records
			$logcount = $this->db->getNumRows();

			if ($logcount > 0)
			{
				foreach ($records as $record)
				{
					$this->log->incrementLinenumber();

					foreach ($exportFields as $field)
					{
						$fieldName = $field->field_name;

						// Set the field value
						$fieldValue = '';

						if (isset($record->$fieldName))
						{
							$fieldValue = $record->$fieldName;
						}

						// Store the field value
						$this->fields->set($field->csvi_templatefield_id, $fieldValue);
					}

					// Output the data
					$this->addExportFields();

					// Output the contents
					$this->writeOutput();
				}
			}
			else
			{
				$this->addExportContent(\JText::_('COM_CSVI_NO_DATA_FOUND'));

				// Output the contents
				$this->writeOutput();
			}
		}
	}
}
