<?php
/**
 * @package     CSVI
 * @subpackage  Productbuilder
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

/**
 * Groups table.
 *
 * @package     CSVI
 * @subpackage  Productbuilder
 * @since       6.6.0
 */
class ProductbuilderTableQuantity extends CsviTableDefault
{
	/**
	 * Table constructor.
	 *
	 * @param   string     $table   Name of the database table to model.
	 * @param   string     $key     Name of the primary key field in the table.
	 * @param   JDatabase  &$db     Database driver
	 * @param   array      $config  The configuration parameters array
	 *
	 * @since   4.0
	 */
	public function __construct($table, $key, &$db, $config = array())
	{
		parent::__construct('#__pb_quant_group', 'group_id', $db, $config);
	}

	/**
	 * Check if all needed data is set.
	 *
	 * @return  bool  True on success | False on failure.
	 *
	 * @since   6.6.0
	 *
	 * @throws  InvalidArgumentException
	 * @throws  RuntimeException
	 */
	public function check()
	{
		$groupId = $this->get('group_id', 0);

		if (0 === (int) $groupId)
		{
			throw new InvalidArgumentException(JText::_('COM_CSVI_GROUP_ID_MISSING'));
		}

		// Check if the group ID exists
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('group_id'))
			->from($this->db->quoteName($this->_tbl))
			->where($this->db->quoteName('group_id') . ' = ' . (int) $groupId);
		$this->db->setQuery($query);

		$existingGroupId = $this->db->loadResult();

		if (!$existingGroupId)
		{
			// We have a group ID, let's add a placeholder entry into the table
			$query->clear()
				->insert($this->db->quoteName($this->_tbl))
				->columns($this->db->quoteName('group_id'))
				->values($groupId);
			$this->db->setQuery($query)->execute();
		}

		return true;
	}

	/**
	 * Reset the primary key.
	 *
	 * @return  boolean  Always returns true.
	 *
	 * @since   6.0
	 */
	public function reset()
	{
		parent::reset();

		// Empty the primary key
		$this->set('group_id', null);

		return true;
	}
}
