<?php
/**
 * @package     CSVI
 * @subpackage  RsForm
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

JFormHelper::loadFieldClass('list');

/**
 * Select list with RSForm! forms.
 *
 * @package     CSVI
 * @subpackage  RsForm
 * @since       6.6.0
 */
class CsviFormFieldRsforms extends JFormFieldList
{
	/**
	 * Type of field
	 *
	 * @var    string
	 * @since  4.0
	 */
	protected $type = 'Rsforms';

	/**
	 * A database connector.
	 *
	 * @var    JDatabase
	 * @since  6.0
	 */
	protected $db = null;

	/**
	 * Method to instantiate the form field object.
	 *
	 * @param   object  $form  The form to attach to the form field object.
	 *
	 * @since   6.0
	 */
	public function __construct($form = null)
	{
		parent::__construct($form);

		$this->db = JFactory::getDbo();
	}

	/**
	 * Get the options.
	 *
	 * @return  array  An array of forms.
	 *
	 * @since   4.0
	 */
	protected function getOptions()
	{
		$query = $this->db->getQuery(true)
			->select($this->db->quoteName('FormId', 'value'))
			->select($this->db->quoteName('FormTitle', 'text'))
			->from($this->db->quoteName('#__rsform_forms'));
		$this->db->setQuery($query);
		$options = $this->db->loadObjectList();

		if (empty($options))
		{
			$options = array();
		}

		return array_merge(parent::getOptions(), $options);
	}
}
