<?php
/**
 * @package     CSVI
 * @subpackage  Plugin.Textinput
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

/**
 * Text input plugin.
 *
 * @package     CSVI
 * @subpackage  Plugin.Text input
 *
 * @since       7.12.0
 */
class PlgCsviextTextInput extends RantaiPluginDispatcher
{
	/**
	 * The unique ID of the plugin
	 *
	 * @var    string
	 * @since  7.12.0
	 */
	private $id = 'textinput';

	/**
	 * JDatabase handler
	 *
	 * @var    JDatabase
	 * @since  7.12.0
	 */
	protected $db = null;

	/**
	 * Construct the class.
	 *
	 * @since   7.12.0
	 */
	public function __construct()
	{
		// Set the dependencies
		$this->db = JFactory::getDbo();
	}

	/**
	 * Method to check if Product cart variant plugin installed
	 *
	 * @param   string  $plugin  The plugin value
	 *
	 * @return   bool  true if plugin is installed | False if plugin not installed.
	 *
	 * @since   7.12.0
	 */
	public function checkIfPluginInstalled($plugin)
	{
		if ($this->id === $plugin)
		{
			return true;
		}

		return false;
	}

	/**
	 * Export product attribute values.
	 *
	 * @param   array   $optionValues  The product attribute values.
	 * @param   string  $fieldType     Field type
	 *
	 * @return  Mixed  List of values to export, False otherwise.
	 *
	 * @since   7.12.0
	 */
	public function exportProductAttribute($optionValues, $fieldType)
	{
		$pluginKey = array_keys($optionValues);

		// Make sure the value exist
		if (!isset($pluginKey[0]))
		{
			return false;
		}

		$pluginKeyValue = $pluginKey[0];
		$values         = array();

		if (is_array($optionValues[$pluginKeyValue]) && isset($optionValues[$pluginKeyValue]['comment']))
		{
			$carVariantValue = $optionValues[$pluginKeyValue]['comment'];
			$values          = \JText::_($fieldType[1]) . ' - ' . $carVariantValue;
		}

		return $values;
	}
}
