<?php
/**
 * @package     CSVI
 * @subpackage  Plugins
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - 2020 RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

defined('_JEXEC') or die;

/**
 * Plugin for triggering a template after the HikaShop order has been created.
 *
 * @package     CSVI
 * @subpackage  Plugins
 * @since       6.5.0
 */
class plgHikashopHikashopcsviexport extends JPlugin
{
	/**
	 * Affects constructor behavior. If true, language files will be loaded automatically.
	 *
	 * @var    boolean
	 * @since  6.5.0
	 */
	protected $autoloadLanguage = true;

	/**
	 * Application object.
	 *
	 * @var    JApplicationCms
	 * @since  6.5.0
	 */
	protected $app;

	/**
	 * Trigger a CSVI export with the given order ID.
	 *
	 * @param   object  &$order       The order  details.
	 * @param   bool    &$send_email  Set if email should be send.
	 *
	 * @return  bool  True on success | False on failure.
	 *
	 * @since   6.5.0
	 */
	public function onAfterOrderCreate(&$order, &$send_email)
	{
		// Don't run from administrator
		if ($this->app->isAdmin())
		{
			return true;
		}

		// Initialise a basic logger with no options (once only).
		jimport('joomla.log.log');
		JLog::addLogger(array(), JLog::ERROR);

		try
		{
			$http = JHttpFactory::getHttp(null, array('curl', 'stream'));
		}
		catch (RuntimeException $e)
		{
			JLog::add($e->getMessage(), JLog::ERROR, 'csviexport');

			return false;
		}

		// Create the URL
		$uri = new JUri(JUri::root() . 'index.php');
		$uri->setVar('option', 'com_csvi');
		$uri->setVar('csvi_template_id', $this->params->get('templateId'));
		$uri->setVar('exportto', 'template');
		$uri->setVar('task', 'export.export');
		$uri->setVar('tmpl', 'component');
		$uri->setVar('form_orderlist', $order->order_id);
		$uri->setVar('key', $this->params->get('secret'));

		if ($this->params->get('filename', false))
		{
			$uri->setVar('form_export_filename', $order->order_id . '.' . $this->params->get('extension', 'csv'));
		}

		// Call the URL
		try
		{
			$rest = $http->get($uri->toString());
		}
		catch (Exception $e)
		{
			JLog::add($e->getMessage(), JLog::ERROR, 'csviexport');

			return false;
		}

		// Check the result code
		if ($rest->code !== 200)
		{
			// Write an error to the error log
			JLog::add(JText::sprintf('PLG_HIKASHOP_HIKASHOPCSVIEXPORT_EXPORT_ERROR', $uri->toString(), $rest->code), JLog::ERROR, 'csviexport');

			return false;
		}

		return true;
	}
}
